//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Программная среда AVR
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if !defined(RT_HW_CORE_DEFINED) && defined(RT_HW_CORE_AVR)
#define      RT_HW_CORE_DEFINED        RT_HW_CODE_CORE_AVR		//--Идентификатор программной среды;
//----------------------1.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE        		RT_HW_CODE_ARCH_AVR			//--Код архитектуры;
#define RT_HW_ARCH_NAME        		"AVR"						//--Наименование архитектуры;
#define RT_HW_CORE_NAME        		"AVR"	 					//--Наименование программной среды
//----------------------1.2.Дополнительные макросы-------------------------------------------------
#define RT_HW_IRAM_ATTR											//--Атрибут для совместимости c ESP8266;	
#define	RT_HW_BLOCK_SET_RESOLUTION_ADC							//--Блокировка функции установка разрешения ADC;
#define RT_HW_BLOCK_SET_RESOLUTION_PWM							//--Блокировка функции установка разрешения PWM;
//----------------------1.3.Общие библиотеки-------------------------------------------------------
#include <avr/pgmspace.h>										//--Библиотека для работы с PROGMEM;
//----------------------1.4.Интерфейсы UART--------------------------------------------------------
#include <SoftwareSerial.h>										//--Библиотека программного UART;
//----------------------1.5.Интерфейсы i2c---------------------------------------------------------
#include <Wire.h>												//--Подключение библиотеки i2c;	
#define RT_HW_I2C_SPEED_DEFAULT  	RT_HW_I2C_SPEED_FM			//--Код скорости i2c по умолчанию (->400kHz);
#define RT_HW_I2C_BLOCK_USER_PINS	1							//--Блокировка изменения пинов;
//----------------------1.6.Интерфейсы SPI---------------------------------------------------------
#include <SPI.h>												//--Подключение библиотеки SPI	
#define RT_HW_SPI_SPEED				RT_HW_SPI_SPEED_ST			//--Скорость SPI в кГц; 
#define RT_HW_SPI_BIT_ORDER         MSBFIRST					//--Порядок следования битов в байте SPI;
#define RT_HW_SPI_DATA_MODE         SPI_MODE0					//--Режим для импульсов SPI;
#define RT_HW_SPI_INTERRUPT_MODE	0							//--Режим работы с прерываниями SPI;
#define RT_HW_SPI_BLOCK_USER_PINS	1							//--Блокировка изменения пинов;
#define RT_HW_SPN_DELAY				0							//--Задержка для RT_HW_PIN_DIR_WRITE_HIGH->0=0ns;1=600ns;2=650ns;3=700ns;4=850ns;5=900ns;
																//  Для справки: RT_HW_PIN_DIR_WRITE_HIGH=300ns;
//----------------------1.7.Макросы для работы c пинами через функции WIRING-----------------------  
#define RT_HW_PIN_MODE_INPUT  					INPUT			//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  					OUTPUT			//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  			INPUT_PULLUP 	//--Дискретный вход с подтяжкой к +Vcc;
//----------------------1.8.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  			INPUT			//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  			INPUT			//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  			INPUT			//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  		OUTPUT			//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------1.9.Базовый размер переменной и структура для GPIO---------------------------
#define RT_HW_DATE_TYPE 		uint8_t						
struct  RT_HW_PIN_DIR_ID {uint8_t pin; uint8_t dir=0; uint8_t bit; volatile uint8_t *base;};	
//----------------------1.10.Макросы прямого управления пинами----------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	    	id.pin=p; id.bit=digitalPinToBitMask(p);\
                                                      id.base=portInputRegister(digitalPinToPort(p));
#define RT_HW_PIN_DIR_MODE_INPUT(id)        *((id.base)+(1)) &=~id.bit; *((id.base)+(2))&=~id.bit; 
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    *((id.base)+(1)) &=~id.bit; *((id.base)+(2))|= id.bit;
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    *((id.base)+(1)) |= id.bit;
#define RT_HW_PIN_DIR_READ(id)           (((*((id.base)+(0))) & id.bit) ? (1) : (0)) 
#define RT_HW_PIN_DIR_WRITE(id,val) if(val){*((id.base)+(2)) |= id.bit;}\
                                       else{*((id.base)+(2)) &=~id.bit;};
#define RT_HW_PIN_DIR_WRITE_LOW(id)         *((id.base)+(2)) &=~id.bit;
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        *((id.base)+(2)) |= id.bit;
//-------------------------------------------------------------------------------------------------	
#endif			
//=================================================================================================
