//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Программная среда ESP8266
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if !defined(RT_HW_CORE_DEFINED) && defined(RT_HW_CORE_ESP8266)
#define RT_HW_CORE_DEFINED             RT_HW_CODE_CORE_ESP8266	//--Идентификатор программной среды;
//----------------------2.1.Основные параметры-----------------------------------------------------
#define	RT_HW_ARCH_CODE        		RT_HW_CODE_ARCH_ESP8266		//--Код архитектуры;
#define RT_HW_ARCH_NAME        		"ESP8266"					//--Наименование архитектуры;
#define RT_HW_CORE_NAME        		"ESP8266"	 				//--Наименование программной среды
//----------------------2.2.Дополнительные макросы-------------------------------------------------
#define RT_HW_IRAM_ATTR				IRAM_ATTR					//--Атрибут для совместимости c ESP8266;
#define	RT_HW_BLOCK_SET_RESOLUTION_ADC							//--Блокировка функции установка разрешения ADC;
#define RT_HW_BLOCK_SET_RESOLUTION_PWM							//--Блокировка функции установка разрешения PWM;
//----------------------2.3.Общие библиотеки-------------------------------------------------------
#include <pgmspace.h>											//--Работа с PROGMEM;
//----------------------2.4.Интерфейсы UART--------------------------------------------------------
#include <SoftwareSerial.h>										//--Библиотека программного UART;
#define RT_HW_BLOCK_UART1_READ									//--Блокировка функций чтения с UART1;	
//----------------------2.5.Интерфейсы i2c---------------------------------------------------------
#include <Wire.h>												//--Подключение библиотеки i2c;
#define RT_HW_I2C_SPEED_DEFAULT  	RT_HW_I2C_SPEED_ST			//--Код скорости i2c по умолчанию (->100kHz);
#define RT_HW_I2C_BLOCK_USER_PINS	0							//--Блокировка изменения пинов i2c;
//----------------------2.6.Интерфейсы SPI---------------------------------------------------------
#include <SPI.h>												//--Подключение библиотеки SPI;
#define RT_HW_SPI_SPEED				RT_HW_SPI_SPEED_ST			//--Скорость SPI в Гц; 
#define RT_HW_SPI_BIT_ORDER         MSBFIRST					//--Порядок следования битов в байте SPI;
#define RT_HW_SPI_DATA_MODE         SPI_MODE0					//--Режим для импульсов SPI;
#define RT_HW_SPI_INTERRUPT_MODE	0							//--Режим работы с прерываниями SPI;
//#define RT_HW_SPI_BLOCK_USER_PINS	0							//--Блокировка изменения пинов SPI;
#define RT_HW_SPN_DELAY			0							//--Задержка для RT_HW_PIN_DIR_WRITE_HIGH->??? 0=0ns;1=600ns;2=650ns;3=700ns;4=850ns;5=900ns;
																//  Для справки: RT_HW_PIN_DIR_WRITE_HIGH=??? 300ns;
//----------------------2.7.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  					INPUT			//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  					OUTPUT			//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  			INPUT_PULLUP 	//--Дискретный вход с подтяжкой к +Vcc;
//----------------------2.8.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  			INPUT			//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  			INPUT			//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  			INPUT			//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  		OUTPUT			//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------2.9.Базовый размер переменной и структура для GPIO---------------------------
#define RT_HW_DATE_TYPE 		uint32_t		 				//--Тип базовой переменной при прямом управлении пинами;
struct  RT_HW_PIN_DIR_ID{uint32_t pin; uint32_t dir=0; uint32_t GPFFS_pin; uint32_t LEFT_PIN;  uint32_t LEFT_GPCI; uint32_t LEFT_GPCD; uint32_t LEFT_GPFPU;};
//----------------------2.10.Макросы прямого управления пинами--------------------------------------
static inline __attribute__((always_inline))		//--Чтение дискретного пина
bool RT_HW_BASE_digitalReadFast(RT_HW_PIN_DIR_ID &id) {if(id.pin < 16){return GPIP(id.pin);} else if(id.pin == 16){return GP16I & 0x01;} return 0;};
#define RT_HW_PIN_DIR_SET_ID(id,p)        id.pin=p; id.GPFFS_pin=GPFFS(GPFFS_GPIO(id.pin)); id.LEFT_PIN=(1<<id.pin); id.LEFT_GPCI=(0xF<<GPCI);\
                                                    id.LEFT_GPCD=(1<<GPCD); id.LEFT_GPFPU=(1<<GPFPU);
#define RT_HW_PIN_DIR_MODE_INPUT(id)      GPF(id.pin) =id.GPFFS_pin; GPEC=id.LEFT_PIN; GPC(id.pin)=(GPC(id.pin) & id.LEFT_GPCI) | (id.LEFT_GPCD);
#define RT_HW_PIN_DIR_MODE_PULLUP(id)     GPF(id.pin) =id.GPFFS_pin; GPEC=id.LEFT_PIN; GPC(id.pin)=(GPC(id.pin) & id.LEFT_GPCI) | (id.LEFT_GPCD); GPF(id.pin)|=id.LEFT_GPFPU;
#define RT_HW_PIN_DIR_MODE_OUTPUT(id)     GPF(id.pin)=id.GPFFS_pin; GPC(id.pin)=GPC(id.pin) & id.LEFT_GPCI; GPES = id.LEFT_PIN;
#define RT_HW_PIN_DIR_READ(id)            RT_HW_BASE_digitalReadFast(id)
#define RT_HW_PIN_DIR_WRITE_LOW(id)       if(id.pin<16){GPOC=id.LEFT_PIN;} else if(id.pin==16){GP16O |=  1;}
#define RT_HW_PIN_DIR_WRITE_HIGH(id)      if(id.pin<16){GPOS=id.LEFT_PIN;} else if(id.pin==16){GP16O &= ~1;}
#define RT_HW_PIN_DIR_WRITE(id,val)       if(id.pin<16){if(val){GPOS=id.LEFT_PIN;} else{GPOC=id.LEFT_PIN;}} else if(id.pin==16){if(val){GP16O |= 1;} else{GP16O &= ~1;}}
#define DIRECT_WRITE_LOW(base, mask)    (GPOC = (mask))             //GPIO_OUT_W1TC_ADDRESS
#define DIRECT_WRITE_HIGH(base, mask)   (GPOS = (mask))             //GPIO_OUT_W1TS_ADDRESS
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
