//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						20.Программная среда CORE_STM32_M_F1
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if !defined(RT_HW_CORE_DEFINED) && defined(RT_HW_CORE_STM32_M_F1)
#define      RT_HW_CORE_DEFINED        RT_HW_CODE_CORE_STM32_M_F1 //--Идентификатор программной среды;
//----------------------20.1.Основные параметры----------------------------------------------------
#define RT_HW_ARCH_CODE        		RT_HW_CODE_ARCH_STM32F1		//--Код архитектуры;
#define RT_HW_ARCH_NAME        		"STM32F1"					//--Наименование архитектуры;
#define RT_HW_CORE_NAME        		"STM32_M_F1"	 			//--Наименование программной среды
//----------------------20.2.Дополнительные макросы-------------------------------------------------
#define RT_HW_IRAM_ATTR											//--Атрибут для совместимости c ESP8266;
#define	RT_HW_BLOCK_SET_RESOLUTION_ADC							//--Блокировка функции установка разрешения ADC;
#define RT_HW_BLOCK_SET_RESOLUTION_PWM							//--Блокировка функции установка разрешения PWM;
//----------------------20.3.Общие библиотеки------------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//----------------------20.4.Интерфейсы UART-------------------------------------------------------
#include <usb_serial.h>											//--Библиотека поддержки USB;
//----------------------20.5.Интерфейсы i2c--------------------------------------------------------
#include <SoftWire.h>
#include <Wire.h>												//--Подключение библиотеки i2c;
#define RT_HW_I2C_SPEED_DEFAULT  	RT_HW_I2C_SPEED_FM			//--Код скорости шины i2c (default 400kHz);	
#define RT_HW_I2C_BLOCK_USER_PINS	1							//--Блокировка изменения пинов i2c;
#define RT_HW_I2C_SOFT_DELAY_STANDART 19						//--Задержка (в циклах asm nop) для стандартной скорости;
#define RT_HW_I2C_SOFT_DELAY_FAST      0						//--Задержка (в циклах asm nop) для быстрой скорости;
#define RT_HW_I2C_SOFT_DELAY_HSPEED    0						//--Задержка (в циклах asm nop) для высокой скорости;
#define RT_HW_BLOCK_I2C_FLUSH									//--Блокировка функции Wirex.flush();
//#include <SoftWire.h>  	
//SoftWire SWire(PB6,PB7,SOFT_FAST); // SOFT_FAST);
//----------------------20.6.Интерфейсы SPI--------------------------------------------------------
#include <SPI.h>												//--Подключение библиотеки SPI;	
#define RT_HW_SPI_SPEED				RT_HW_SPI_SPEED_ST			//--Скорость SPI в Гц; 
#define RT_HW_SPI_BIT_ORDER         MSBFIRST					//--Порядок следования битов в байте SPI;
#define RT_HW_SPI_DATA_MODE         SPI_MODE0					//--Режим для импульсов SPI;
#define RT_HW_SPI_INTERRUPT_MODE	0							//--Режим работы с прерываниями SPI;
//#define RT_HW_SPI_BLOCK_USER_PINS	1							//--Блокировка изменения пинов SPI;
#define RT_HW_SPN_DELAY				1							//--Задержка для RT_HW_PIN_DIR_WRITE_HIGH->??? 0=0ns;1=600ns;2=650ns;3=700ns;4=850ns;5=900ns;
																//  Для справки: RT_HW_PIN_DIR_WRITE_HIGH=??? 300ns;
//----------------------20.7.Макросы для работы c пинами через функции WIRING----------------------
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------20.8.Макросы для совместимости с STM32-------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT_ANALOG		//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM			 PWM				//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN PWM_OPEN_DRAIN		//--ШИМ выход с открытым коллектором;	
//----------------------20.9.Базовый размер переменной и структура для GPIO------------------------
#define RT_HW_DATE_TYPE 		uint32_t 						//--Тип базовой переменной при прямом управлении пинами;
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir=0; uint32_t bit; volatile uint32_t *base;}; //--Структура для прямого управления пинами
//----------------------20.10.Макросы прямого управления пинами-------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p)	  id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)  pinMode(id.pin,INPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_PULLUP(id) pinMode(id.pin,INPUT_PULLUP);	//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) pinMode(id.pin,OUTPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_OUTPUT_PP, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_READ(id)        digitalRead(id.pin)			//digitalReadFast((PinName)id.pin);  //digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val)   digitalWrite(id.pin,val);		//if(val){digitalWriteFast((PinName)id.pin,HIGH);} else {digitalWriteFast((PinName)id.pin,LOW);} 
#define RT_HW_PIN_DIR_WRITE_LOW(id)   digitalWrite(id.pin,LOW);		//digitalWriteFast((PinName)id.pin, LOW); 	//digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)  digitalWrite(id.pin,HIGH);	//digitalWriteFast((PinName)id.pin, HIGH);	//digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
