//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						5.Программная среда PI PICO
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if !defined(RT_HW_CORE_DEFINED) && defined(RT_HW_CORE_PI_PICO)
#define RT_HW_CORE_DEFINED             RT_HW_CODE_CORE_PI_PICO 	//--Идентификатор программной среды;
//----------------------5.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE        		RT_HW_CODE_ARCH_PI_PICO		//--Код архитектуры;
#define RT_HW_ARCH_NAME        		"Raspberry Pi Pico"			//--Наименование архитектуры;
#define RT_HW_CORE_NAME        		"Raspberry Pi Pico"	 		//--Наименование программной среды
//----------------------5.2.Дополнительные макросы-------------------------------------------------
#define RT_HW_IRAM_ATTR											//--Атрибут для совместимости c ESP8266;	
//#define RT_HW_BLOCK_SET_RESOLUTION_ADC							//
//#define RT_HW_BLOCK_AVAILABLE_FOR_WRITE
//----------------------2.3.Общие библиотеки-------------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
#include "pico/stdlib.h"										//--SDK
//----------------------5.4.Интерфейсы UART--------------------------------------------------------
//----------------------5.5.Интерфейсы i2c---------------------------------------------------------
#include <Wire.h>												//--Подключение библиотеки i2c;
#define RT_HW_I2C_SPEED_DEFAULT  	RT_HW_I2C_SPEED_FM			//--Код скорости шины i2c (default 400kHz);
#define RT_HW_I2C_BLOCK_USER_PINS	0							//--Блокировка изменения пинов i2c;
//----------------------5.6.Интерфейсы SPI---------------------------------------------------------
#include <SPI.h>
#define RT_HW_SPI_SPEED				RT_HW_SPI_SPEED_ST			//--Скорость SPI в Гц; 
#define RT_HW_SPI_BIT_ORDER         MSBFIRST					//--Порядок следования битов в байте SPI;
#define RT_HW_SPI_DATA_MODE         SPI_MODE0					//--Режим для импульсов SPI;
#define RT_HW_SPI_INTERRUPT_MODE	0							//--Режим работы с прерываниями SPI;
#define RT_HW_SPI_BLOCK_USER_PINS	0							//--Блокировка изменения пинов SPI;
#define RT_HW_SPN_DELAY				1							//--Задержка для RT_HW_PIN_DIR_WRITE_HIGH->??? 0=0ns;1=600ns;2=650ns;3=700ns;4=850ns;5=900ns;
																//  Для справки: RT_HW_PIN_DIR_WRITE_HIGH=??? 300ns;
//----------------------5.7.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  					INPUT			//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  					OUTPUT			//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  			INPUT_PULLUP 	//--Дискретный вход с подтяжкой к +Vcc;
//----------------------5.8.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  			INPUT			//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  			INPUT			//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  			INPUT			//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  		OUTPUT			//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------5.9.Базовый размер переменной и структура для GPIO-------------------------
#define RT_HW_DATE_TYPE 		uint32_t						//--Тип базовой переменной при прямом управлении пинами;
#pragma pack(push,1)
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir=0; uint32_t bit; volatile uint32_t *base;};
#pragma pack(pop)
//----------------------5.10.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	    	id.pin=p; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT); 				
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    pinMode(id.pin,OUTPUT);
#define RT_HW_PIN_DIR_READ(id)           	digitalRead(id.pin) 
#define RT_HW_PIN_DIR_WRITE(id,val) 		digitalWrite(id.pin,val);
#define RT_HW_PIN_DIR_WRITE_LOW(id)         digitalWrite(id.pin,LOW);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        digitalWrite(id.pin,HIGH);
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================