//=================================================================================================
//                             Вывод на консоль адресов i2c
//-------------------------------------------------------------------------------------------------
//По переднему по переднему фронту eN и всегда первый раз при eN=1 выводит на консоль 
//  адреса устройств i2c в десятичной(DEC), шестнадцатичной(HEX) или комбинированной формах.
//Примечание: если консоль не настроена, то выполняется настройка консоли и
//  вывод выполняется после готовности консоли. 
//ecoins@mail.ru 01.11.2021г.
//=================================================================================================
// AVR  	- одна шина i2c(Wire),  объединенная библиотека master/slave;
// ESP8266  - одна шина i2c(Wire),  объединенная библиотека master/slave;
// ESP32    - две  шины i2c master(Wire1,Wire2), библиотека master - Wire.h, 
//            при использовании библиотеки slave - ESP32_I2C_Slave (требуется загрузка)
//            две шины i2с slave(WireSlave1, WireSlave2.
//            *** одновременно можно использовать только две шины в разном сочетании master/slave; 
// SAM3     - одна шина i2c(Wire),  объединенная библиотека master/slave;
// Seeeduino- до 6 шин  i2c(Wire,Wire1-5), объединенная библиотека master/slave;
// STM32    - до 3 шин  i2c(Wire определена, 
//                          Wire1 нужно определять TwoWire Wire1(RT_HW_I2C1_SDA,  RT_HW_I2C1_SCL),
//                          Wire2 нужно определять TwoWire Wire2(RT_HW_I2C2_SDA,  RT_HW_I2C2_SCL);
//			  для STM32duino     - объединенная библиотека master/slave;
//			  для STM32master_F1 - библиотека master - Wire.h, библиотека slave - Wire_slave.h (в core STM32master), 
//			  для STM32master_F4 - библиотека master - Wire.h, библиотека slave - отсутствует;
//	номера битов device.i2cx.state:
//  bit.0=RT_HW_I2C_STATE_INIT, bit.1=RT_HW_I2C_STATE_BUSY,       bit.2=RT_HW_I2C_STATE_MASTER, bit.3=RT_HW_I2C_STATE_SLAVE, 
//  bit.4=RT_HW_I2C_STATE_MODE, bit,5=RT_HW_I2C_STATE_BLOCK_PINS, bit.6=RT_HW_I2C_STATE_FREE1,  bit.7=RT_HW_I2C_STATE_FREE2;					 
//------------------------------------------------------------------------------------------------- 
// Функция i2cFindDevice() Осуществляет поиск устройства по его адресу на шине i2c.
//  Поиск принимается успешным при многократном (подряд) обнаружении устройства с периодом 50мс. 
//  Вход: addr-адрес устройств; bus -номер щины i2c;
//  	  id.custom - состояние функции. Перед вызовом функции установить =0;
//		  id.maxOk  - кол-во успешных повторов, после которого устройство считается обнаруженным (1-15) (default=3); 
//		  id.maxErr - максимальное количество ошибок (1-255) (default).
//                    При ошибках оределяет период повторного поиска (=100*id.cntErr); 	
//  Если устройство с заданным адресом не найдено, то период поиска увеличается от 100мс до (100 х cntMax).
//	Если устройство найдено, то делаются дополнительные запросы через 50мс (всего должно быть успешных подряд поисков >id.maxOk);
//  Выход: id.custom;
//struct RT_HW_STRUCT_I2C_FIND{uint8_t custom=0; uint8_t maxErr=20; uint8_t maxOk=3; uint8_t cntErr:5, cntOk:3; uint32_t timeDelay;};
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//										API вспомогательных функций i2c
// 							num - логический номер шины; bus - системный  номер шины; 
//      struct RT_HW_STRUCT_I2C {uint8_t state; uint8_t sda,scl; uint8_t adr; uint32_t speed;}; 
// enum {RT_HW_I2C_STATE_INIT=0, RT_HW_I2C_STATE_MASTER=1, RT_HW_I2C_STATE_SLAVE=2, RT_HW_I2C_STATE_BUSY=6, RT_HW_I2C_STATE_MODE=7};
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//=================================================================================================
//								1.СОЗДАНИЕ ОБЪЕКТОВ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.СОЗДАНИЕ ОБЪЕКТОВ С ДАННЫМИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_I2C0)										//--1.0.1.Данные для i2c0			  
RT_HW_STRUCT_I2C 	LinkI2C0;										
#endif	
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)										//--1.1.1.Данные для i2c1
RT_HW_STRUCT_I2C 	LinkI2C1;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C2)										//--1.2.1.Данные для i2c0
RT_HW_STRUCT_I2C 	LinkI2C2;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2CS)										//--1.8.1.Данные для i2cS (программная)
RT_HW_STRUCT_I2C 	LinkI2CS;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2CS1)										//--1.9.1.Данные для i2cS1 (программная)
RT_HW_STRUCT_I2C 	LinkI2CS1;										
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									1.Определение объектов в .cpp
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//=================================================================================================
//// 				2.Функции проверки номера шины, адреса, управления регистром состояния                       
//=================================================================================================
//uint8_t i2cNumToBus     (uint8_t num);									//--Конвертация логического номера i2c в физический номер шины i2c;
bool 	i2cCheckBus     (             					uint8_t bus=0);	//--Проверка наличия шины с физическим номером bus;
uint8_t i2cInitMaster   (								uint8_t	bus=0);	//--Настройка шины как Master;
void    i2cSetBitState  (          		uint8_t numBit, uint8_t bus=0);	//--Установка бита состояния;
void    i2cClearBitState(          		uint8_t numBit, uint8_t bus=0);	//--Сброс     бита состояния;
void    i2cWriteBitState(bool 	 val, 	uint8_t numBit, uint8_t bus=0);	//--Запись 	  бита состояния;
bool 	i2cGetBitState	(          		uint8_t numBit, uint8_t bus=0);	//--Проверка  бита состояния;
void    i2cSetState     (uint8_t val,              		uint8_t bus=0);	//--Установка бита состояния;
void    i2cSetStatus    (uint8_t val,              		uint8_t bus=0);	//--Установка статуса;
void    i2cSetCodeErr   (uint8_t val,              		uint8_t bus=0);	//--Установка ошибки;
uint8_t i2cGetState  	(				           		uint8_t bus=0);	//--Получение слова состояния шины;
uint8_t i2cGetStatus  	(				           		uint8_t bus=0);	//--Получение статуса;	
uint8_t i2cGetCodeErr  	(				           		uint8_t bus=0);	//--Получение кода ошибки;	
//=================================================================================================
// 							3.Запись/чтение номеров пинов, Slave-адреса, скорости
//=================================================================================================
void    i2cSetPinSCL    (uint8_t scl,   				uint8_t bus=0);	//--Запись в регистр номера пинa SCL;
void    i2cSetPinSDA    (uint8_t sda,   				uint8_t bus=0);	//--Запись в регистр номера пина SDA;
uint8_t i2cGetPinSCL    (			    				uint8_t bus=0);	//--Запись в регистр номера пинa SCL;
uint8_t i2cGetPinSDA    (	         					uint8_t bus=0);	//--Запись в регистр номера пина SDA;
void    i2cSetAdr       (uint8_t adr,   				uint8_t bus=0);	//--Запись в регистры адреса (=0:Master; >0:Slave);
uint8_t i2cGetAdr 		(               				uint8_t bus=0);	//--Чтение адреса шина (из регистра);
void    i2cSetSpeed		(uint32_t speed,				uint8_t bus=0);	//--Запись частоты(кГц)(из регистра);
uint32_t i2cGetSpeed	(               				uint8_t bus=0);	//--Чтение частоты(кГц)(из регистра);
void    i2cSetClock		(								uint8_t bus=0); //--Настройка частоты шины i2c(частота из регистра);
void    i2cSetClockNew	(uint32_t clock, 				uint8_t bus=0);
//=================================================================================================
//											5.Функции обмена
//=================================================================================================
void     i2cBeginTransmission(uint8_t adr,				uint8_t bus=0);	//--Захват устр с адресом=adr;
uint8_t  i2cEndTransmission(			  				uint8_t bus=0);	//--Завершение передачи,иниц.функцией write();
void     i2cFlush(						  				uint8_t bus=0);	//--Очистка буферов и индексов;
void     i2cWrite(uint8_t val,							uint8_t bus=0);	//--Запись байта  для отправки в i2c;
void     i2cWrite(const uint8_t *arr, uint8_t qnt,  	uint8_t bus=0);	//--Запись qnt байт     из массива arr в буфер для отправки в i2c;
void     i2cWrite(const    char *arr, uint8_t qnt,  	uint8_t bus=0);	//--Запись qnt символов из массива arr в буфер для отправки в i2c;	
uint8_t  i2cAvailable(									uint8_t bus=0);	//--Проверка данных в приемном буфере;
uint8_t  i2cRead(                           			uint8_t bus=0); //--Чтение байта из буфера i2c;
uint8_t  i2cRequestFrom(bool sendStop,uint8_t qnt, uint8_t adr, uint8_t bus=0); //==Запрос данных у Slave;
//=================================================================================================
//											6.Производные Функции обмена с устройством
//=================================================================================================
uint8_t  i2cSendByte (uint8_t val,                     uint8_t adr, uint8_t bus=0);	//--Запись байта; 
uint8_t  i2cSendBytes(const uint8_t *arr, uint8_t qnt, uint8_t adr, uint8_t bus=0);	//--Запись qnt   байт из arr[];
uint8_t  i2cGetBytes (bool sendStop,      uint8_t qnt, uint8_t adr, uint8_t bus=0);	//--Чтение qnt   байт в буфер i2c;
uint8_t  i2cGetByte  (                                 uint8_t adr, uint8_t bus=0);	//--Чтение байта в буфер i2c;	   
uint8_t  i2cReadRegs (uint8_t reg,        uint8_t qnt, uint8_t adr, uint8_t bus=0);	//--Чтение qnt байт начиная регистра=reg в буфер i2c;
//=================================================================================================
//											7.Сервисные функции
//=================================================================================================
uint8_t i2cFindAdr(uint32_t &timeDelay,uint16_t &custom,uint8_t okMax,uint8_t period,uint16_t periodMax,uint8_t adr,uint8_t bus=0); //--Поиск устройства adr;			
uint8_t i2cFindAdr(uint32_t &timeDelay,uint16_t &custom,uint8_t adr,uint8_t bus); 	//--6.2.Поиск устройства adr(c фиксированными параметрами 3,20,2000);	
void    i2cHeadInterface(uint8_t bus=0);										//--6.3.Вывод заголовка по интерфейсу;
void    i2cScanBus(uint8_t bus=0); 													//--6.4.Сканирование шины i2c;								
//=================================================================================================
//							20.Инициализация шины i2c
//=================================================================================================
void     i2cBegin	  (uint8_t bus=0);			//==Инициализация шины i2c как Master;
void     i2cBeginSlave(uint8_t bus=0);			//==Инициализация шины i2c как slave;
//=================================================================================================
