//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР  DHT22
// 1.Линия сенсора: притягивается к земле (600mсs); переводится в режим чтения; последовательно принимается 83 изменения линии.
//	 Если нет линия не меняется более 90mcs - фиксируется ошибка с кодом 2.
//    FALLING, RISING и FALLING - это стартовый бит, затем 40 бит: 
// 2. Проверяется контрольная сумма ---> codeErr=3.
// 3. Проверяется диапазон (температура -50.0/+125.0 градусов по Цельсию, влажность 0/100.0 %);
// 4. При отсутствии ошибок запоминаются полученные результаты и при необходимости взводится флаг изменившихся данных (id.fresh);
// 5. После  ошибочных чтений, превышающих maxQntErr, результатам T и H присваивается DHTx_NAN_INT
// Коды ошибок: 
// (=0)-нет ошибок;(=201)-пин не настроен(=255);(=202)-пин без функции DI;(=203) пин нет функции DO;
// (=100-184)-ошибка по тайм-ауту 90msc импульсов с данными;
// (=205)-ошибка контрольной суммы; (=206)-измеренные показания за пределами диапазонов измерений;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_DHT22_VER  	 		11		//--Номер версии библиотеки;
#define RT_HW_DHT22_DELAY_ERR   	500 	//--Базовая задержка при повторов после ошибки;
#define RT_HW_DHT22_DELAY_ERR_MIN	500		//--Минимальный период при ошибках;
#define RT_HW_DHT22_DELAY_ERR_MAX	2500	//--Максимальный период при ошибках;
#define RT_HW_DHT22_TIME_SENSOR     1200		//--Время цикла измерения внутри сенсора после запуска измерения;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 							Управление (mode=1 - "жесткая" задержка после импульса запуска)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void dht22(RT_HW_STRUCT_DHT22_ID &z){		//==Управление чтением сенсора;
//=================================================================================================
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==0) {z.ok=0; z.run=z.fresh=1; z.err=0;							//--Подготовка флагов; 
			  z.vTemp=-0.1; z.vHum=0.1;							//--Начальные значения;
              z.codeErr=z.stepDelay=0; z.dir=20; return;}				//--Подготовка параметров и переход;
//-------------------------------------------------------------------------------------------------
// 							Ожидание флага запуска
//-------------------------------------------------------------------------------------------------			  
if(z.dir==20){if(!z.run){return;}
              z.run=0; z.codeErr=0; z.step=0; z.dir++;} 	//==Ожидание флага запуска;				  
//-------------------------------------------------------------------------------------------------
// 							Проверка/настройка текущего пина
//-------------------------------------------------------------------------------------------------	
if(z.dir==21){dht22_checkPin(z); 							//--Проверка доступности пина; 
			  if(z.err){z.dir=0; return;}					//--Выход по фатальной ошибке;
			  if(z.action)      {return;} 					//--Ожидание завершения настройки пина;
			  z.step=0; z.dir=30;return;}					//--Переход на измерение;
//-------------------------------------------------------------------------------------------------
// 							Измерение
//-------------------------------------------------------------------------------------------------	
if(z.dir==30){dht22_Measurument(z);		   							//--Измерение;
			  if(z.err){z.dir=0; return;} 							//--Выход по ошибке;
			  if(z.action)      						{return;} 	//--Ожидание завершения измерения;
			  z.sizeDelay=100;
			  z.dir=80;          						return;}	//--Переход на основной цикл;		
//-------------------------------------------------------------------------------------------------
// 					Задержка
//-------------------------------------------------------------------------------------------------
if(z.dir==80){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} 
				z.dir=20; return;}
//-------------------------------------------------------------------------------------------------
} //--End dht22(RT_HW_STRUCT_DHT22_ID &z);
//=================================================================================================
//					Проверка пина
//=================================================================================================
void dht22_checkPin(RT_HW_STRUCT_DHT22_ID &z){
if(z.step==0){z.action=1; z.err=0;  z.step++;}
if(z.step==1){if(z.extPin==255)  {z.err=1; z.codeErr=10; return;}			//--Выход, если блокирующий пин не настроен;
			  if(z.pin==z.extPin){z.action=0;            return;}
			  z.pin=z.extPin;  												//--Проверка пина;
			  if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,z.pin)){z.err=1; z.codeErr=11; return;} //--Выход по ошибке, если пин нe DI;
			  if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z.pin)){z.err=1; z.codeErr=12; return;} //--Выход по ошибке, если пин не DO;
			  digitalWrite(z.pin,1); pinMode(z.pin,OUTPUT); z.step++;}
if(z.step==2){if(!timeDelay(z.timeBegin,z.stepDelay,2)) {return;} 
		      z.action=0; return;}					
}									//--Настройка пина и выход;
//=================================================================================================
//					Измерение параметров
//=================================================================================================
void dht22_Measurument(RT_HW_STRUCT_DHT22_ID &z){
if(z.step==0) {z.action=1; z.err=0; z.step++;}
if(z.step==1) {z.timeStart=micros(); z.step++;}
if(z.step==2) {digitalWrite(z.pin,LOW); pinMode(z.pin,OUTPUT); digitalWrite(z.pin,LOW); //--Притягивание линии устройства к земле;
			  // if(z.modeDelay){delayMicroseconds(RT_HW_DHT22_TIME_SENSOR);} 		//--"Жесткая" задержка;
			   //else  {if(getPastMcs(z.timeStart)<RT_HW_DHT22_TIME_SENSOR){return;}}	//--"Мягкая"  задержка;
			   delayMicroseconds(RT_HW_DHT22_TIME_SENSOR);// Send start signal
               digitalWrite(z.pin, HIGH);
			   pinMode(z.pin, INPUT); 							 	//--Switch bus to receive data
			   z.step++;}											//--Переход на измерение;
//-------------------------------------------------------------------------------------------------
if(z.step==3) {uint16_t rawH=0; uint16_t rawT=0; uint16_t data=0; uint32_t startTime; int16_t vT; uint16_t vH;
			   RT_HW_PIN_DIR_ID idPin; RT_HW_PIN_DIR_SET_ID(idPin,z.pin);
			   //----------------------------------------------------------------------------------
			   // ESP32 is a multi core/multi processing chip.
			   // It is necessary to disable task switches during the readings
			   //----------------------------------------------------------------------------------
			   if(!z.blockInterrupts){
#if defined(RT_HW_CORE_ESP8266) || defined(RT_HW_CORE_ESP32) || defined(RT_HW_CORE_ESP32_C3) 
			   noInterrupts();	
#endif
			   }//----------------------------------------------------------------------------------
			   //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			   //	          Цикл фиксации 83 входных импульсов (первые три -стартовых импульса):
			   //1.cначала FALLING, RISING и FALLING фронт для стартового бита
			   //2.затем 40 бит: НАРАЩИВАЮЩИЙСЯ и затем СПАДАЮЩИЙ фронт на бит
			   //  Чтобы сделать наш код простым, принимаем любое значение HIGH или LOW, 
			   //                                                    если оно не превышает 100 мкс.
			   //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++			
			   for(int8_t i=-3 ; i<2 * 40; i++){uint8_t age; startTime = micros();
				  do{age=(uint32_t)(micros()-startTime);
			        if(age>100){z.codeErr=20+i; 
					z.sizeDelay=500; z.stepDelay=0; 
					z.step=51; return;}}
				  while(RT_HW_PIN_DIR_READ(idPin)==(i & 1) ? HIGH : LOW);
				 // while(digitalRead(z.pin)==(i & 1) ? HIGH : LOW);
	           //----------------------------------------------------------------------------------
			   if((i>=0)&&(i&1)){data<<=1; if(age>30){data|=1;}}
			   //----------------------------------------------------------------------------------
			   // Разбор принятых импульсов по параметрам
			   //----------------------------------------------------------------------------------
			   if(i==31){rawH=data;}						//--Фиксация данных по влажности;
			   if(i==63){rawT=data; data = 0;}				//--Фиксация данных по температуре;
			   } //--Завершение цикла for(int8_t i=-3; i<80; i++)----------------------------------			 
			   //----------------------------------------------------------------------------------
			   //		Восстановление прерываний (по пользовательскому параметру) 
			   //----------------------------------------------------------------------------------
			   if(!z.blockInterrupts){
#if defined(RT_HW_CORE_ESP8266) || defined(RT_HW_CORE_ESP32) || defined(RT_HW_CORE_ESP32_C3) 
			   interrupts();
#endif
			   }//---------------------------------------------------------------------------------
			   //----------------------------------------------------------------------------------
			   //		Проверка на контрольную сумму
			   //----------------------------------------------------------------------------------
			   if((uint8_t)(((uint8_t)rawH)+(rawH>>8)+((uint8_t)rawT)+(rawT>>8))!= data){z.codeErr=200; z.step=50; return;}
			   //----------------------------------------------------------------------------------
			   //				Обработка результата измерения
			   //----------------------------------------------------------------------------------
               if(rawT & 0x8000 ) {vT = -(int16_t)(rawT & 0x7FFF);} else {vT=rawT;}
			   vH=rawH;
			   //----------------------------------------------------------------------------------
			   //		Обработка и проверка результата измерения
			   //----------------------------------------------------------------------------------		       
			   if(vH > 1000){z.codeErr=201; z.step=50; return;}
			   if(vT < -500){z.codeErr=202; z.step=50; return;}
			   if(vT > 1250){z.codeErr=203; z.step=50; return;}
			   //----------------------------------------------------------------------------------
			   //		Успешное завершение
			   //----------------------------------------------------------------------------------			   
			   z.cnt=0; z.ok=z.fresh=1; z.vTemp=vT/10.0; z.vHum=vH/10.0; 
			   z.run=0; z.sizeDelay=1200; z.action=0; z.step=0;
			   } 
//-------------------------------------------------------------------------------------------------
// 					Динамическая задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
if(z.step==50){   z.sizeDelay=RT_HW_DHT22_DELAY_ERR_MIN+z.cnt*RT_HW_DHT22_DELAY_ERR; 
			   if(z.sizeDelay>RT_HW_DHT22_DELAY_ERR_MAX) {z.sizeDelay=RT_HW_DHT22_DELAY_ERR_MAX;} z.step++; return;}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay))                        {return;} 
			   if(++z.cnt>=z.qntMaxErr){z.cnt=z.qntMaxErr; z.run=0; z.err=1; z.codeErr=15; return;}
																				 z.step=1; return;}
//-------------------------------------------------------------------------------------------------
// 					Задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
//if(z.step==60){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
//-------------------------------------------------------------------------------------------------
// 					Задержка с последующим успешным выходом
//-------------------------------------------------------------------------------------------------
//if(z.step==70){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} 
//				z.action=0; return;}
//-------------------------------------------------------------------------------------------------
}
//=================================================================================================