//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР  DS1820
// 1.Линия сенсора: притягивается к земле (600mсs); переводится в режим чтения; последовательно принимается 83 изменения линии.
//	 Если нет линия не меняется более 90mcs - фиксируется ошибка с кодом 2.
//    FALLING, RISING и FALLING - это стартовый бит, затем 40 бит: 
// 2. Проверяется контрольная сумма ---> codeErr=3.
// 3. Проверяется диапазон (температура -50.0/+125.0 градусов по Цельсию, влажность 0/100.0 %);
// 4. При отсутствии ошибок запоминаются полученные результаты и при необходимости взводится флаг изменившихся данных (id.fresh);
// 5. После  ошибочных чтений, превышающих maxQntErr, результатам T и H присваивается DHTx_NAN_INT
// Коды ошибок: 
// (=0)-нет ошибок;(=201)-пин не настроен(=255);(=202)-пин без функции DI;(=203) пин нет функции DO;
// (=100-184)-ошибка по тайм-ауту 90msc импульсов с данными;
// (=205)-ошибка контрольной суммы; (=206)-измеренные показания за пределами диапазонов измерений;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_DS1820_VER          15	//--Версия библиотеки DS1820;
#define RT_HW_DS1820_TIME_SENSOR  800	//--Время цикла измерения внутри сенсора;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			Чтение
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void ds1820(RT_HW_STRUCT_ONE_WIRE &iw, RT_HW_STRUCT_DS1820_ID &id, uint8_t mode=1){	//==Управление чтением сенсора;
//=================================================================================================
if(id.dir==0){
id.run=id.fresh=1; 
id.ok=0;
id.vT=RT_HW_DEFAULT_T_NAN; 								//--Установка начальных состояний;
id.codeErr=0; id.cntErrCur=0; id.maxQntErr=3;			//--Параметры ошибок измерений;
id.cntOk=id.cntErr=0;									//--Очистка счетчиков наработки;
id.blockInterrupts=0;									//--Очистка флага блокировки прерываний;
id.dir++;}
//=================================================================================================
//					По команде запуска(run) проверка входных параметров и запуск измерения
//=================================================================================================
if(id.dir==1){
if(!id.run) {return;}															//--Ожидание команды запуска;
uint8_t ok=1; id.codeErr=0;														//--Подготовка параметров;
	      if(id.extPin==255)							  {ok=0; id.codeErr=201;}	//--Выход, если пин не настроен;
if(ok==1){if(id.pin!=id.extPin){id.pin=id.extPin; ok=2;}}				//--Проверка номера пина на изменение;
if(ok==2){if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,id.extPin)){ok=0; id.codeErr=202;}	//--Выход по ошибке, если пин нe DI;
          if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,id.extPin)){ok=0; id.codeErr=203;}}	//--Выход по ошибке, если пин нe DO;
if(ok==0){return;}																//--Выход по ошибке настройки; 
if(ok==2){oneWireBegin(iw,id.pin);}												//--Инициализация шины;
id.dir++;																		//--Переход на следующий этап;
}
//=================================================================================================
//					Поэтапное Измерение параметров
//=================================================================================================
if(id.dir==2){
oneWireReset(iw);						//--Сброс шины;
if(iw.state==0){return;}				//--Ожидание завершения сброса;
if(iw.state==1){id.dir++;}				//--После сброса обнаружены устройства;
if(iw.state>=2){id.dir=20; return;}}	//--Устройства не обнаружены; переход на обработку ошибок;
//=================================================================================================
if(id.dir==3){oneWireWrite(iw,0xCC); id.dir++; return;}
//=================================================================================================
if(id.dir==4){oneWireWrite(iw,0x44); id.timeBegin=micros(); id.dir++; return;}
//=================================================================================================
if(id.dir==5){	
if(mode==1) {delayMicroseconds(RT_HW_DS1820_TIME_SENSOR);} 						//--"Жесткая" задержка;
else        {if(getPastMcs(id.timeBegin)<RT_HW_DS1820_TIME_SENSOR) {return;}}	//--Ожидание завершения "мягкой" задержки;
id.dir++;} 										//--Переход на следующий этап;
//=================================================================================================
if(id.dir==6){
oneWireReset(iw);
if(iw.state==0){return;}						//--Ожидание завершения сброса;
if(iw.state==1){id.dir++;}						//--После сброса обнаружены устройства;
if(iw.state>=2){id.codeErr=201; id.dir=20;}}	//--Устройства не обнаружены; переход на обработку ошибок;
//=================================================================================================
if(id.dir==7){oneWireWrite(iw,0xCC); id.dir++; return;}
//=================================================================================================
if(id.dir==8){oneWireWrite(iw,0xBE); id.dir++;}
//=================================================================================================
if(id.dir==9){
uint8_t data[9]; uint8_t crc; int16_t raw; uint8_t ok=1;
oneWireReadBytes(iw,data,9);
crc=crc8(data,8); if((crc !=data[8]) ||  (data[4]==0))  {iw.state=5; id.codeErr=202; ok=0; id.dir=20;}
//=================================================================================================
if(ok){
raw = (data[1] << 8) | data[0];										//---Вычисление измеренного значения;
if(id.typeSensor){raw=raw <<3; if (data[7]==0x10) {raw=(raw & 0xFFF0)+12-data[6];}}
else			 {uint8_t cfg = (data[4] & 0x60);
	if      (cfg==0x00){raw = raw & ~7;} //--Урезаем, если 9 bit resolution, 93.75 ms
	else if (cfg==0x20){raw = raw & ~3;} //--Урезаем, если 10 bit res, 187.5 ms
	else if (cfg==0x40){raw = raw & ~1;} //--Урезаем, если 11 bit res, 375 ms
				 }
id.ok=1; id.cntErrCur=0; if(id.vT!=raw) {id.vT=raw; id.fresh=1;}}
//-------------------------------------------------------------------------------------------------
id.run=0;
if(ok){id.cntOk++; id.dir=1; return;} else {id.cntErr++; id.dir=20;}}  
//=================================================================================================
if(id.dir==20){id.run=0; id.dir=1; if(++id.cntErrCur>id.maxQntErr) {id.cntErrCur=0; id.ok=0; id.fresh=1; id.vT=RT_HW_DEFAULT_T_NAN;}}	
}
//-------------------------------------------------------------------------------------------------
