 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР MAX6675
/*
 Сенсор MAX6675 410 / 5000
 MAX6675 выполняет компенсацию холодного спая и оцифровывает сигнал термопары типа К.
 Данные выводятся с 12-битным разрешением, совместимым с SPI, используется только линия чтения miso.
 Точность измерения до 0,25 ° C, позволяет измерять температуру в диапазоне 0 + 1023.75°C.
 Точность с 12-битным разрешением обеспечивается в диапазоне от 0 ° C до + 700°C, 
 за границами этого дипазона точность в последнем разряде не гарантируется.
 
 MAX6675:
	1.это сложная термопара, преобразованная в цифровой преобразователь со встроенным 12-битным АЦП;
	2.имеет холодный спай, компенсацию измерения и коррекции, цифровой контроллер.
	3.имеет интерфейс, совместимый с SPI, и связанную с ним логику управления.
	4.разработан для работы с внешним микроконтроллером).
 
 Преобразование температуры включает в себя оборудование формирования сигнала для
 преобразоваybz сигнал термопары в напряжение совместимое со встроенным АЦП. 
 Т+(хромовый вывод термопары) и Т-(алюминевый вывод теромопары) подключаются к внутренней схеме MAX6675.
 Перед преобразованием термоэлектрических напряжений в эквивалентные значения температуры, необходимо компенсировать
 разницу  между стороной холодного спая термопары(температура окружающей среды чипа MAX6675) и виртуальным эталон 0°C.
 
 Для термопары типа K изменение напряжения равно 41 мкВ/°C, 
 что приблизительно соответствует характеристике термопары со следующей линейным уравнением:
		VOUT = (41 мкВ /°C) ✕ (TR - TAMB), 
 где TAMB - температура окружающей среды чипа MAX6675.
 Чип MAX6675 измеряет температуру окружающей среды на встроенноим диоде.
 Наиболее точные показания получается достигаются когда холодный спай термопары и корпус чипа MAX6675 имеют
 одинаковую температуру.
 К оцифровке АЦП чип добавляет измерение температуры на встроенном диоде (холодный спай) и 
 считывает 12-разрядный результат на вывод SO.
 Следует избегать размещения тепловыделяющих устройств или компонентов рядом с чипом, 
 поскольку это может привести к ошибкам из-за температуры холодного спая.
 Последовательность всех 0, означает, что температура спая равна 0°C, последовательность всех единиц соответствует 1023.75°C.
 Библиотека обеспечивает отфильтровывание ошибочных измерений (от 0 до 7, по умолчанию =3); 
 
 Коды ошибок:
 0 - нет ошибок;
 60 - недопустимый номер шины    (для SSPI);
 61 - недопустимыЙ номер пина cs (для SSPI); 
 62 - SPI не настроен            (для SPI);
 65 - недопустимый номер пина sck(для SSPI);
 67 - недопустимый номер пина so (для SSPI); 
 68 - недопустимый номер пина cs (для SSPI); 
 71 - сенсор не подключен;
 72 - ошибочное измерение;
 73 - обрыв термопары;
*/ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_MAX6675_VER 			12		//--Номер версии библиотеки;
#define RT_HW_MAX6675_RESOLUTION    0.25   	//--Разрешение измерения в градусах С;
#define RT_HW_MAX6675_DELAY_ERR    	50   	//--Задержка при однократной ошибке;
#define RT_HW_MAX6675_DELAY_ERR_MIN	250		//--Минимальный период при ошибках;
#define RT_HW_MAX6675_DELAY_ERR_MAX	2500	//--Максимальный период при ошибках;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//					Инициализация для SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void max6675(RT_HW_STRUCT_MAX6675_ID &z){
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==0) {z.ok=0;  z.run=z.fresh=1;							//--Подготовка флагов; 
			  z.vTemp=-0.1;										//--Начальные значения;
			  z.codeErr=z.stepDelay=0; 
			  if(!z.mi){z.codeErr=spiInitDevice(z.cs,z.bus);} 			//--err:bus->60;cs->61;SPI.begin()->62; //--Настройка SPI;
			  else     {z.codeErr=SSPIinitDevice(z.sck,z.so,255,z.cs);} //--err:sck->65,so->67,cs->68;		  	//--Настройка SSPI;
			  if(z.codeErr){z.dir=90;} else{z.dir=20;} return;}			  
//-------------------------------------------------------------------------------------------------
// 							Ожидание флага запуска
//-------------------------------------------------------------------------------------------------																				
if(z.dir==20){if(!z.run) {return;} z.run=0; z.step=0; z.dir=30;}											//--Ожидание флага запуска;
//-------------------------------------------------------------------------------------------------
// 							Измерение
//-------------------------------------------------------------------------------------------------	
if(z.dir==30){max6675_Measurument(z);	if(z.err){z.dir=0; return;} 	//==Пошаговое измерение и переход по фатальной ошибке;
			  if(z.action)      						  {return;} 	//--Ожидание завершения измерения;
			  z.dir=20;          						   return;}		//--Переход на основной цикл;		
} //--End max6675(RT_HW_STRUCT_MAX6675_ID &z);

//=================================================================================================
//					Измерение параметров
//=================================================================================================
void max6675_Measurument(RT_HW_STRUCT_MAX6675_ID &z){
if(z.step==0) {z.action=1; z.err=0; z.step++;}
if(z.step==1) {if(!z.mi){z.rawData=SPItransfer16(z.clock,MSBFIRST,SPI_MODE0,0,z.cs,z.bus);}//--Чтение показаний через SPI;
			   else 	{z.rawData=SSPIread16(z.time,MSBFIRST,z.sck,z.so,z.cs);}	//--Чтение показаний через SSPI;	
				    if(z.rawData==0xffff)   {z.codeErr=71;}							//--Датчик не подключен;
			   else if(z.rawData==0) 	    {z.codeErr=72;}							//--Ошибочное измерение;
			   else if(bitRead(z.rawData,2)){z.codeErr=73;}							//--Обрыв термопары;														 
			   else   					    {z.codeErr=0;} 							//--Измерение без ошибок;
			   if(z.codeErr){z.step=50; return;}										//--Выход на обработку ошибки;
			   //----------------------------------------------------------------------------------
			   //		Успешное завершение
			   //----------------------------------------------------------------------------------			   									
			   z.vTemp=(float)(z.rawData>>3) * RT_HW_MAX6675_RESOLUTION; 													
			   z.action=0; z.cnt=0; z.ok=z.fresh=1;
			  }
//-------------------------------------------------------------------------------------------------
// 					Динамическая задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
if(z.step==50){z.sizeDelay=RT_HW_MAX6675_DELAY_ERR_MIN+z.cnt*z.cnt*RT_HW_MAX6675_DELAY_ERR; 
			  if(z.sizeDelay>RT_HW_MAX6675_DELAY_ERR_MAX) {z.sizeDelay=RT_HW_MAX6675_DELAY_ERR_MAX;} z.step++; return;}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} 
			  if(++z.cnt>=z.qntMaxErr){z.cnt=z.qntMaxErr; z.err=1; z.codeErr=50;} else{z.step=1;} return;}
}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
// 					Динамическая задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
//if(z.dir==50){if(!z.cnt){z.cnt=1;} z.sizeDelay=z.cnt*z.cnt*RT_HW_MAX6675_DELAY_ERR; z.dir++;}	//--Контроль повторных ошибок;
//if(z.dir==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} 
//			  if(++z.cnt>=z.qntMaxErr){z.err=1; z.cnt=z.qntMaxErr; z.dir=0;} else{z.dir=20;} return;}
//-------------------------------------------------------------------------------------------------
//}												 
//=================================================================================================	
