//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			Sensor BMP180
// default adr i2c=0x77
// 0 = Success
// 1 = Data too long to fit in transmit buffer
// 2 = Received NACK on transmit of address
// 3 = Received NACK on transmit of data
// 4 = Other error
// Oversampling: 0 to 3, higher numbers are slower, higher-res outputs (->delay=5,8,14,26).
// Will return delay in ms to wait, or 0 if I2C error.
// REG_CONTROL=0xF4; REG_RESULT=0xF6;  CMND_TEMP=0x2E; 
// CMND_PRESS0=0x34; CMND_PRESS1=0x74; CMND_PRESS2=0xB4; CMND_PRESS3=0xF4;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_BMP180_VER      	11		//--Номер версии библиотеки;
#define RT_HW_BMP180_DELAY_ERR  50 		//--Базовая задержка при повторов после ошибки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//							Direct measurement
//=================================================================================================
void bmp180(RT_HW_STRUCT_BMP180_ID &z){
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==0) {z.runTemp=z.runPress=1; z.ok=0; z.fresh=1; 
			  z.vTemp=-0.1; z.vPress= 0.2;				//--Начальные параметры для вывода;
			  z.codeErr=i2cInitMaster(z.bus); 			//--Настройка шины i2c;
			  if(z.codeErr){z.dir=95; return;}			//--Выход по ошибке;
              z.sizeDelay=0; z.dir=5;}						//--Подготовка параметров и переход;
//-------------------------------------------------------------------------------------------------
// 							Поиск устройства на шине i2c
//-------------------------------------------------------------------------------------------------
if(z.dir==5) {if(i2cFindAdr(z.timeBegin,z.sizeDelay,z.adr,z.bus)){z.dir=15;} return;}
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==15){z.step=0; z.dir++;}			//--Настройка параметров;
if(z.dir==16){bmp180_parameters(z);			//--Пошаговая настройка; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения;
			  z.dir=20;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Ожидание флагов запуска измерений
//-------------------------------------------------------------------------------------------------
if(z.dir==20){if(z.runTemp || z.runPress) {z.dir=30;} else {return;}}			
//-------------------------------------------------------------------------------------------------
// 							Измерение температуры
//-------------------------------------------------------------------------------------------------
if(z.dir==30){if(z.runTemp){z.runTemp=0; z.step=0; z.dir++;} else {z.dir=40;}}	//--Проверка флага;
if(z.dir==31){bmp180_MeasurumentTemp(z);	//--Пошаговое измерение;
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения;
			  z.dir=40;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Измерение давления
//-------------------------------------------------------------------------------------------------
if(z.dir==40){if(z.runPress){z.runPress=0; z.step=0; z.dir++;} else {z.dir=80;}}	//--Проверка флага;
if(z.dir==41){bmp180_MeasurumentPress(z);	//--Пошаговое измерение; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения;
			  z.dir=80;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 					Задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
if(z.dir==80){z.stepDelay=0; z.dir++;}
if(z.dir==81){if(!timeDelay(z.timeBegin,z.stepDelay,50)){return;} else {z.dir=20; return;}}
//-------------------------------------------------------------------------------------------------
}
//=================================================================================================
//								Measurement Temp
//=================================================================================================
void bmp180_MeasurumentTemp(RT_HW_STRUCT_BMP180_ID &z){
if(z.step== 0){z.err=0; z.action=1; z.cnt=0; z.step++;}										//--Подготовка
if(z.step== 1){uint8_t arr[2]; arr[0]=0xF4; arr[1]=0x2E;  									//--REG_CONTROL=0xF4;CMND_TEMP=0x2E;						   	
			   z.codeErr=i2cSendBytes(arr,2,z.adr,z.bus); if(z.codeErr){z.step=50; return;}	//--Отправка команды чтения температуры;;
			   z.step++;} 
if(z.step== 2){if(!timeDelay(z.timeBegin,z.stepDelay,5)){return;}							//--Задержка;
			   z.codeErr=i2cReadRegs(0xF6,2,z.adr,z.bus); if(z.codeErr){z.step=50; return;}	//--Запрос чтения 2-байт с регистра REG_RESULT=0xF6;
			   uint8_t arr[2];																//--Создание временного массива; 
			   arr[0]=i2cRead(z.bus); arr[1]=i2cRead(z.bus);								//--Сохранение 2-x байт масссиве;		
			   float tu,a;
			   tu = (arr[0] * 256.0) + arr[1];
			  //example from http://wmrx00.sourceforge.net/Arduino/BMP085-Calcs.pdf  tu = 0x69EC;
			   a = z.c5 * (tu - z.c6);
			   z.vTemp = a + (z.mc / (a + z.md));
			   z.fresh=z.ok=1; z.action=0; return;}				  														   
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BMP180_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
}
//=================================================================================================
//								Measurement Press
//=================================================================================================
void bmp180_MeasurumentPress(RT_HW_STRUCT_BMP180_ID &z){
if(z.step== 0){z.err=0; z.action=1; z.cnt=0; z.step++;}					//--Подготовка
if(z.step== 1){uint8_t arr[2];	  arr[0]=0xF4;			  				//--REG_CONTROL=0xF4
								  arr[1]=0x34; z.delayPress=5; 			//--CMND_PRESS_0=0x34						   
			   if(z.OverPress==1){arr[1]=0x74; z.delayPress=8;}			//--CMND_PRESS_1=0x74
			   if(z.OverPress==2){arr[1]=0xB4; z.delayPress=14;}		//--CMND_PRESS_2=0xB4
			   if(z.OverPress==3){arr[1]=0xF4; z.delayPress=26;}		//--CMND_PRESS_3=0xF4
			   z.codeErr=i2cSendBytes(arr,2,z.adr,z.bus); if(z.codeErr){z.step=50; return;}	//--Отправка команды;
			   z.step++;} 
if(z.step== 2){if(!timeDelay(z.timeBegin,z.stepDelay,z.delayPress)){return;}				//--Задержка;
			   z.codeErr=i2cReadRegs(0xF6,3,z.adr,z.bus); if(z.codeErr){z.step=50; return;}	//--REG_RESULT=0xF6
			   uint8_t arr[3]; 
			   arr[0]=i2cRead(z.bus); arr[1]=i2cRead(z.bus); arr[2]=i2cRead(z.bus);
			   float pu,s,x,y,zu;
			   pu = (arr[0] * 256.0) + arr[1] + (arr[2]/256.0);
			   //example from http://wmrx00.sourceforge.net/Arduino/BMP085-Calcs.pdf, pu = 0x982FC0;	
			   //pu = (0x98 * 256.0) + 0x2F + (0xC0/256.0);		
			   s = z.vTemp - 25.0;
			   x = (z.x2 * pow(s,2)) + (z.x1 * s) + z.x0;
			   y = (z.y2 * pow(s,2)) + (z.y1 * s) + z.y0;
			   zu= (pu - x) / y;
			   z.vPress = (z.p2 * pow(zu,2)) + (z.p1 * zu) + z.p0;
			   z.fresh=z.ok=1; z.action=0; return;}				  														   
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BMP180_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
}

//=================================================================================================
//								Measurement Sea Level
//		double sealevel(double P, double A);
			// convert absolute pressure to sea-level pressure (as used in weather data)
			// P: absolute pressure (mbar)
			// A: current altitude (meters)
			// returns sealevel pressure in mbar

//		double altitude(double P, double P0);
			// convert absolute pressure to altitude (given baseline pressure; sea-level, runway, etc.)
			// P: absolute pressure (mbar)
			// P0: fixed baseline pressure (mbar)
			// returns signed altitude in meters
//double SFE_BMP180::sealevel(double P, double A)
// Given a pressure P (mb) taken at a specific altitude (meters),
// return the equivalent pressure (mb) at sea level.
// This produces pressure readings that can be used for weather measurements.
// Учитывая давление P (мБ), снятое на определенной высоте (в метрах),
// вернуть эквивалентное давление (мБ) на уровне моря.
// Это дает показания давления, которые можно использовать для измерения погоды.
//{
//	return(P/pow(1-(A/44330.0),5.255));
//}


//double SFE_BMP180::altitude(double P, double P0)
// Given a pressure measurement P (mb) and the pressure at a baseline P0 (mb),
// return altitude (meters) above baseline.
// При измерении давления P (мбар) и базовом давлении P0 (мбар),
// возвращаем высоту (в метрах) над базовой линией
//{
//	return(44330.0*(1-pow(P/P0,1/5.255)));
//}
//=================================================================================================
//void bmp180_MeasurumentSeaLevel(RT_HW_STRUCT_BMP180_ID &z){}
//=================================================================================================
//								Read parameters
//=================================================================================================
void bmp180_parameters(RT_HW_STRUCT_BMP180_ID &z){
if(z.step==0) {z.err=0; z.action=1; z.cnt=0; z.step++;}				//--Подготовка;
if(z.step==1) {z.codeErr=i2cReadRegs(0xAA,6,z.adr,z.bus); 
			   if(z.codeErr){z.step=50; return;}					//--Переход на задержку по ошибке обмена;
			   z.AC1=((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
			   z.AC2=((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
			   z.AC3=((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);   
	// int16_t arr[3]; arr[0]=z.AC1; arr[1]=z.AC2; arr[2]=z.AC3; 
	//consoleVar(String("AC1,AC2,AC3=")); consoleArrVar(arr,'E',0,'I',3);
			   z.step++;      return;}
if(z.step==2) {z.codeErr=i2cReadRegs(0xB0,6,z.adr,z.bus); 
			   if(z.codeErr){z.step=50; return;}					//--Переход на задержку по ошибке обмена;
			   z.AC4=((uint16_t)(i2cRead(z.bus)<<8))|(uint16_t)i2cRead(z.bus);
			   z.AC5=((uint16_t)(i2cRead(z.bus)<<8))|(uint16_t)i2cRead(z.bus);
			   z.AC6=((uint16_t)(i2cRead(z.bus)<<8))|(uint16_t)i2cRead(z.bus);
	//uint16_t arr[3]; arr[0]=z.AC4; arr[1]=z.AC5; arr[2]=z.AC6; 
	//consoleVar(String("AC4,AC5,AC6=")); consoleArrVar(arr,'E',0,'U',3);
			   z.step++;      return;}
if(z.step==3) {z.codeErr=i2cReadRegs(0xB6,10,z.adr,z.bus); 
			   if(z.codeErr){z.step=50; return;}					//--Переход на задержку по ошибке обмена;
			   z.VB1=((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
			   z.VB2=((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
			   z.MB= ((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus); 
			   z.MC= ((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
			   z.MD= ((int16_t)(i2cRead(z.bus)<<8)) | i2cRead(z.bus);
	// int16_t arr[5]; arr[0]=z.VB1; arr[1]=z.VB2; arr[2]=z.MB;  arr[3]=z.MC; arr[4]=z.MD;
	//consoleVar(String("VB1,VB2,MB,MC,MD=")); consoleArrVar(arr,'E',0,'U',5);			   
			   z.step++;      return;}			   
if(z.step==4) {z.c3 = 160.0 * pow(2,-15) * z.AC3;
		       z.c4 = pow(10,-3) * pow(2,-15) * z.AC4;
		       z.b1 = pow(160,2) * pow(2,-30) * z.VB1;
			   z.c5 = (pow(2,-15) / 160) * z.AC5;
			   z.step++;      return;}	
if(z.step==5) {z.c6 = z.AC6;
		       z.mc = (pow(2,11) / pow(160,2)) * z.MC;
			   z.md = z.MD / 160.0;
			   z.x0 = z.AC1;
			   z.x1 = 160.0 * pow(2,-13) * z.AC2;
			   z.x2 = pow(160,2) * pow(2,-25) * z.VB2;
			   z.step++;      return;}	
if(z.step==6) {z.y0 = z.c4 * pow(2,15);
		       z.y1 = z.c4 * z.c3;
		       z.y2 = z.c4 * z.b1;
			   z.p0 = (3791.0 - 8.0) / 1600.0;
			   z.p1 = 1.0 - 7357.0 * pow(2,-20);
		       z.p2 = 3038.0 * 100.0 * pow(2,-36);			   
			   z.step++;      return;}		   
if(z.step==7) {z.action=0;    return;}
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BMP180_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
}			   
//=================================================================================================

