//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			Sensor BME280
// 0 = Success
// 1 = Data too long to fit in transmit buffer
// 2 = Received NACK on transmit of address
// 3 = Received NACK on transmit of data
// 4 = Other error
// Oversampling: 0 to 3, higher numbers are slower, higher-res outputs (->delay=5,8,14,26).
// Will return delay in ms to wait, or 0 if I2C error.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_BME280_VER       	11		//--Номер версии библиотеки;
#define RT_HW_BME280_DELAY_ERR 	50 		//--Базовая задержка при повторов после ошибки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//							Direct measurement
//=================================================================================================
void bme280(RT_HW_STRUCT_BME280_ID &z){
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==0) {z.runTemp=z.runPress=1; z.initFresh=0; z.ok=0; z.fresh=1; 
			  z.vTemp=-0.1; z.vPress= 0.2;				//--Начальные параметры для вывода;
			  z.codeErr=i2cInitMaster(z.bus); 			//--Настройка шины i2c;
			  if(z.codeErr){z.dir=95; return;}			//--Выход по ошибке;
              z.sizeDelay=0; z.dir=5;}						//--Подготовка параметров и переход;
//-------------------------------------------------------------------------------------------------
// 							Поиск устройства на шине i2c
//-------------------------------------------------------------------------------------------------
if(z.dir==5) {if(i2cFindAdr(z.timeBegin,z.sizeDelay,z.adr,z.bus)){z.dir=15;} return;}
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(z.dir==15){z.step=0; z.dir++;}			//--Настройка параметров;
if(z.dir==16){bme280_parameters(z);			//--Пошаговая настройка параметров; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения;
			  z.dir=20;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Ожидание флагов запуска измерений
//-------------------------------------------------------------------------------------------------
if(z.dir==20){if(z.runTemp || z.runHum || z.runPress) {z.dir=30; return;} else {return;}}			
//-------------------------------------------------------------------------------------------------
// 							Измерение температуры
//-------------------------------------------------------------------------------------------------
if(z.dir==30){if(z.runTemp) {z.runTemp=0; z.step=0; z.dir++;} else {z.dir=40; return;}}	//--Проверка флага;
if(z.dir==31){bme280_MeasurumentTemp(z);	//--Пошаговое измерение; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения функции калибровки;
			  z.dir=40;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Измерение Влажности
//-------------------------------------------------------------------------------------------------
if(z.dir==40){if(z.runHum)  {z.runHum=0; z.step=0; z.dir++;} else {z.dir=50; return;}}	//--Проверка флага;
if(z.dir==41){bme280_MeasurumentHum(z);		//--Пошаговая настройка параметров; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения функции калибровки;
			  z.dir=50;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Измерение давления
//-------------------------------------------------------------------------------------------------
if(z.dir==50){if(z.runPress){z.runPress=0;z.step=0;z.dir++;} else {z.dir=60; return;}}	//--Проверка флага;
if(z.dir==51){bme280_MeasurumentPress(z);	//--Пошаговая настройка параметров; 
              if(z.err){z.dir=0; return;} 	//--Переход на начало по ошибке;
			  if(z.action)      {return;} 	//--Ожидание завершения функции калибровки;
			  z.dir=60;}					//--Переход;
//-------------------------------------------------------------------------------------------------
if(z.dir==60){z.dir=80;}
//-------------------------------------------------------------------------------------------------
// 					Задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
if(z.dir==80){z.stepDelay=0; z.dir++;}
if(z.dir==81){if(!timeDelay(z.timeBegin,z.stepDelay,50)){return;} else {z.dir=20; return;}}
//-------------------------------------------------------------------------------------------------
}
//=================================================================================================
//								Read parameters
//=================================================================================================
void bme280_parameters(RT_HW_STRUCT_BME280_ID &z){
if(z.step==0) {z.err=0; z.action=1; z.cnt=0; z.step++;}				//--Подготовка;
if(z.step==1) {if(!timeDelay(z.timeBegin,z.stepDelay,50))    {return;} 				    z.step++; return;}	//--Задержка для нормализации питания;
if(z.step==2) {bme280_readByte(z,0xD0); 	 		if(z.codeErr){z.step=50; return;} z.chipID=z.val; z.step++; return;}	//--Чтение кода сенсора;					
if(z.step==3) {     if(z.chipID==0x56) {z.modeChip=1; z.permitHum=0;} 	//--Сенсор типа BMP280;
               else if(z.chipID==0x57) {z.modeChip=1; z.permitHum=0;} 	//--Сенсор типа BMP280;
		       else if(z.chipID==0x58) {z.modeChip=1; z.permitHum=0;} 	//--Сенсор типа BMP280;
			   else if(z.chipID==0x60) {z.modeChip=2; z.permitHum=1;} 	//--Сенсор типа BME280;
               else 	               {z.modeChip=0; z.err=1;return;}	//--Неверный код;	
			   z.step++; return;}	
if(z.step==4) {bme280_writeByte(z,0xE0,0xB6);   	if(z.codeErr){z.step=50; return;} z.step++; return;} //--Программный сброс;	
if(z.step==5) {if(!timeDelay(z.timeBegin,z.stepDelay,10)){return;} 
			   z.step++;}
if(z.step==6) {if(!z.permitTemp){z.step=7; return;};
	           uint8_t data[6];
			   bme280_readBytes(z,0x88,6,data); 	if(z.codeErr){z.step=50; return;}	//--DIG_T1_LSB=0x88,0x89,0x8A,0x8B,0x8C,0x8D;
			   z.dig_T1 = (data[1] << 8) | data[0];
			   z.dig_T2 = (data[3] << 8) | data[2];
			   z.dig_T3 = (data[5] << 8) | data[4];	
			   z.step++; return;}
if(z.step==7) {if(!z.permitPress){z.step=10; return;};
			   uint8_t data[6];
			   bme280_readBytes(z,0x8E,6,data); 	if(z.codeErr){z.step=50; return;}	//--DIG_P1_LSB=0x8E,0x8F,0x90,0x91,0x92,0x93;			   
			   z.dig_P1 = (data[1] << 8) | data[0];
			   z.dig_P2 = (data[3] << 8) | data[2];
			   z.dig_P3 = (data[5] << 8) | data[4];	
			   z.step++; return;}
if(z.step==8) {uint8_t data[6];
			   bme280_readBytes(z,0x94,6,data); 	if(z.codeErr){z.step=50; return;}	//--DIG_P4_LSB=0x94,0x95,0x96,0x97,0x98,0x99;
			   z.dig_P4 = (data[1] << 8) | data[0];
			   z.dig_P5 = (data[3] << 8) | data[2];
			   z.dig_P6 = (data[5] << 8) | data[4];	
			   z.step++; return;}
if(z.step==9) {uint8_t data[6];
			   bme280_readBytes(z,0x9A,6,data); 	if(z.codeErr){z.step=50; return;}	//--DIG_P7_LSB=0x9A,0x9B,0x9C,0x9D,0x9E,0x9F;
			   z.dig_P7 = (data[1] << 8) | data[0];
			   z.dig_P8 = (data[3] << 8) | data[2];
			   z.dig_P9 = (data[5] << 8) | data[4];	
			   z.step++; return;}
if(z.step==10){if(!z.permitHum){z.step=12; return;};
			   bme280_readByte(z,0xA1); 	    	if(z.codeErr){z.step=50; return;} 	
			   z.dig_H1=z.val; 
			   z.step++; return;}	//--DIG_H1=0xA1;
if(z.step==11){uint8_t data[7];
			   bme280_readBytes(z,0xE1,7,data); 	if(z.codeErr){z.step=50; return;}	//--DIG_H2_LSB=0x9A,0x9B,0x9C,0x9D,0x9E,0x9F;
			   z.dig_H2 =(data[1] << 8) | data[0];
			   z.dig_H3 = data[2] << 8;
	           z.dig_H4 =(data[3] << 4) |(data[4] &0x0F);
			   z.dig_H5 =(data[5] << 4) |((data[4] >> 4) & 0x0F);
			   z.dig_H6 = data[6];
			   z.step++; return;}
if(z.step==12){z.val=z.osrs_h & 0b00000111;																	//--Настройка CTRL_HUM;
				bme280_writeByte(z,0xF2,z.val);		if(z.codeErr){z.step=50; return;} z.step++; return;} 	//--Запись в регистр CTRL_HUM;  
if(z.step==13){z.val=(z.t_sb   << 5) | (z.filter << 2) | z.spi3w_en;										//--Настройка CONFIG;
				bme280_writeByte(z,0xF5,z.val);		if(z.codeErr){z.step=50; return;} z.step++; return;}   //--Запись в регистр CONFIG;      
if(z.step==14){z.val=(z.osrs_t << 5) | (z.osrs_p << 2) | z.mode;											//--Настройка CTRL_MEAS;
				bme280_writeByte(z,0xF4,z.val); 	if(z.codeErr){z.step=50; return;} z.step++; return;} 	//--Запись в регистр CTRL_MEAS;
if(z.step==15){bme280_readByte(z,0xD0); 	 		if(z.codeErr){z.step=50; return;}						//--Контрольное чтение chipID; 
                if(z.val!=z.chipID) {z.err=1;  return;} 							  z.step++; return;}	//--Чтение кода сенсора;
//------------------------------------------------------------
if(z.step==16) {z.initFresh=1; z.action=0;    return;}
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BME280_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
//-------------------------------------------------------------------------------------------------
}
//=================================================================================================
//							Чтение температуры
//=================================================================================================
void bme280_MeasurumentTemp  (RT_HW_STRUCT_BME280_ID &z){
if(z.step==0) {z.err=0; z.action=1; z.cnt=0; z.step++; return;}				//--Подготовка;
if(z.step==1) {uint8_t data[3];
			   bme280_readBytes(z,0xFA,3,data); 	if(z.codeErr){z.step=50; return;}	//--temp_xlsb=FC,FB,FA;
			   bme280_readByte(z,0xD0);				if(z.codeErr){z.step=50; return;}
			   if(z.val!=z.chipID){z.codeErr=20;}   if(z.codeErr){z.step=50; return;}
			   z.adc_T  = (uint32_t)data[0] << 12;
			   z.adc_T |= (uint32_t)data[1] <<  4;
			   z.adc_T |= (         data[2] >>  4) & 0x0F;			  
			   z.step++; return;}
if(z.step==2) {int32_t var1, var2; 
				var1 = (int32_t)((z.adc_T / 8) - ((int32_t)z.dig_T1 * 2));
				var1 = (var1 * ((int32_t)z.dig_T2)) / 2048;
				var2 = (int32_t)((z.adc_T / 16) - ((int32_t)z.dig_T1));
				var2 = (((var2 * var2) / 4096) * ((int32_t)z.dig_T3)) / 16384;
				z.t_fine = var1 + var2 + z.t_fine_adjust;
				int32_t T = (z.t_fine * 5 + 128) / 256;
				float vTemp=(float)T / 100; z.vTemp=vTemp;
	//consoleVar(String("adc_T,vTemp.t_fine=")); consoleVar(z.adc_T,','); consoleVar(z.vTemp,';'); consoleVar(z.t_fine,'E');
			    z.fresh=z.ok=1; z.action=0; return;}		
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BME280_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
};
//=================================================================================================
//								Measurement Hum
//=================================================================================================
void bme280_MeasurumentHum(RT_HW_STRUCT_BME280_ID &z){
if(z.step==0) {z.err=0; z.action=1; z.cnt=0; z.step++;}				//--Подготовка
if(z.step==1) {uint8_t data[2];
			   bme280_readBytes(z,0xFD,2,data); 	if(z.codeErr){z.step=50; return;}	//--hum_lsb=FC,FD,FE;
			   bme280_readByte(z,0xD0);				if(z.codeErr){z.step=50; return;}
			   if(z.val!=z.chipID){z.codeErr=20;}   if(z.codeErr){z.step=50; return;}
			   z.adc_H  =(uint32_t)data[0] << 8;
			   z.adc_H |=(uint32_t)data[1]; 
			   if (z.adc_H == 0x8000){z.codeErr=21; z.step=50; return;}
			   z.step++; return;}
if(z.step==2) {int32_t var1, var2, var3, var4, var5;
				var1 = z.t_fine - ((int32_t)76800);
				var2 = (int32_t)(z.adc_H * 16384);
				var3 = (int32_t)(((int32_t)z.dig_H4) * 1048576);
				var4 = ((int32_t)z.dig_H5) * var1;
				var5 = (((var2 - var3) - var4) + (int32_t)16384) / 32768;
				var2 = (var1 * ((int32_t)z.dig_H6)) / 1024;
				var3 = (var1 * ((int32_t)z.dig_H3)) / 2048;
				var4 = ((var2 * (var3 + (int32_t)32768)) / 1024) + (int32_t)2097152;
				var2 = ((var4 * ((int32_t)z.dig_H2)) + 8192) / 16384;
				var3 = var5 * var2;
				var4 = ((var3 / 32768) * (var3 / 32768)) / 128;
				var5 = var3 - ((var4 * ((int32_t)z.dig_H1)) / 16);
				var5 = (var5 < 0 ? 0 : var5);
				var5 = (var5 > 419430400 ? 419430400 : var5);
				uint32_t H = (uint32_t)(var5 / 4096);
				float   vHum=(float)H / 1024.0;
				z.vHum=vHum;
   // consoleVar(String("adc_H,vHum=")); consoleVar(z.adc_H,','); consoleVar(z.vHum,'E');
			    z.fresh=z.ok=1; z.action=0; return;}			   
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BME280_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
}
//=================================================================================================
//								Measurement Press
//=================================================================================================
void bme280_MeasurumentPress(RT_HW_STRUCT_BME280_ID &z){
if(z.step== 0){z.err=0; z.action=1; z.cnt=0; z.step++;}				//--Подготовкаif(z.step== 1){uint8_t arr[2]; arr[0]=0xF4; arr[1]=0x2E;  			//--REG_CONTROL=0xF4;CMND_TEMP=0x2E;						   	
if(z.step== 1){uint8_t data[3];
			   bme280_readBytes(z,0xF7,3,data); 	if(z.codeErr){z.step=50; return;}	//--hum_lsb=FC,FD,FE;
			   bme280_readByte(z,0xD0);				if(z.codeErr){z.step=50; return;}
			   if(z.val!=z.chipID){z.codeErr=20;}   if(z.codeErr){z.step=50; return;}			   
			   z.adc_P  = (uint32_t)data[0] << 12;
 			   z.adc_P |= (uint32_t)data[1] << 4;
			   z.adc_P |= (         data[2] >> 4) & 0x0F;
			   z.step++; return;}
if(z.step== 2){int64_t var1, var2, P;
			   var1 = ((int64_t)z.t_fine) - 128000;
			   var2 =  var1 *   var1 * (int64_t)z.dig_P6;
			   var2 =  var2 + ((var1 * (int64_t)z.dig_P5)<<17);
			   var2 =  var2 + (((int64_t)z.dig_P4)<<35);
			   var1 =((var1 *   var1 * (int64_t)z.dig_P3)>>8) + ((var1 * (int64_t)z.dig_P2)<<12);
			   var1 =(((((int64_t)1)<<47)+var1))*((int64_t)z.dig_P1)>>33;
			   if (var1 == 0){z.action=0; return;}
			   P = 1048576 - z.adc_P;
			   P = (((P << 31) - var2)*3125)/var1;
			   var1 = (((int64_t)z.dig_P9) * (P >> 13) * (P >> 13)) >> 25;
			   var2 = (((int64_t)z.dig_P8) * P) >> 19;
			   P = ((P + var1 + var2) >> 8) + (((int64_t)z.dig_P7)<<4);
			   P = P >> 8; // /256
			   z.vPress=P/100.0;
	//consoleVar(String("adc_P,vPress=")); consoleVar(z.adc_P,','); consoleVar(z.vPress,'E');
			  z.fresh=z.ok=1; z.action=0; return;}	
//-------------------------------------------------------------------------------------------------
if(z.step==50){if(++z.cnt>z.qntMaxErr){z.err=1; return;} else{z.sizeDelay=z.cnt*z.cnt*RT_HW_BME280_DELAY_ERR; z.step++;}}	//--Контроль повторных ошибок;
if(z.step==51){if(!timeDelay(z.timeBegin,z.stepDelay,z.sizeDelay)){return;} z.step=1; return;}
}
//=================================================================================================
//								Measurement Press
//=================================================================================================
//void bme280_MeasurumentSeaLevel(RT_HW_STRUCT_BME280_ID &z){}


//=================================================================================================
//=================================================================================================
//=================================================================================================



//=================================================================================================
//							Запись байта в регистр устройства
//=================================================================================================
void bme280_writeByte(RT_HW_STRUCT_BME280_ID &z, uint8_t reg, uint8_t val){
//-------------------------------------------------------------------------------------------------
if(z.mi=='I'){uint8_t arr[2]; arr[0]=reg; arr[1]=val;												
			  z.codeErr=i2cSendBytes(arr,2,z.adr,z.bus);
			  return;}
//-------------------------------------------------------------------------------------------------
if(z.mi=='S'){z.codeErr=0;
			  return;}
//-------------------------------------------------------------------------------------------------
if(z.mi=='s'){z.codeErr=0;
			  return;}												//--Выход из SSPI;
//-------------------------------------------------------------------------------------------------
}
//=================================================================================================
//							Чтение байта из регистр
//=================================================================================================
void bme280_readByte(RT_HW_STRUCT_BME280_ID &z, uint8_t reg){
//--------------------------------------------------------------------------------------------------
if(z.mi=='I'){z.codeErr=i2cReadRegs(reg,1,z.adr,z.bus); if(z.codeErr){return;}
			  z.val    =i2cRead(z.bus); return;} 			
//-------------------------------------------------------------------------------------------------
if(z.mi=='S'){z.codeErr=0; 
			  return;}	
//-------------------------------------------------------------------------------------------------
if(z.mi=='s'){z.codeErr=0;
			  return;}
//-------------------------------------------------------------------------------------------------
}	
//=================================================================================================
//							Чтение байтn из регистр
//=================================================================================================
void bme280_readBytes(RT_HW_STRUCT_BME280_ID &z, uint8_t reg, uint8_t qnt, uint8_t *data){
//--------------------------------------------------------------------------------------------------
if(z.mi=='I'){z.codeErr=i2cReadRegs(reg,qnt,z.adr,z.bus); if(z.codeErr){return;}
			  for(uint8_t i=0; i<qnt; i++){data[i]=i2cRead(z.bus);}
			  return;} 			
//-------------------------------------------------------------------------------------------------
if(z.mi=='S'){z.codeErr=0; 
			  return;}	
//-------------------------------------------------------------------------------------------------
if(z.mi=='s'){z.codeErr=0;
			  return;}
//-------------------------------------------------------------------------------------------------
}

//=================================================================================================
//							Чтение компенсации температуры
//=================================================================================================
float bme280_getTemperatureCompensation(RT_HW_STRUCT_BME280_ID &z) {
return float((z.t_fine_adjust * 5) >> 8) / 100.0;};
//=================================================================================================
//							Установка компенсации температуры
//=================================================================================================
void bme280_setTemperatureCompensation(RT_HW_STRUCT_BME280_ID &z, float adjustment){
z.t_fine_adjust=((int32_t(adjustment * 100) << 8)) / 5;};
		  
//=================================================================================================	
