//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//						Функци отправки данных/инструкций в LCD
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

//=================================================================================================
//								Отправка одного байта в Lcd 
// 1.code: =T отправка тетрады; =I отправка инструкция(байт); =D отправка данных (байт);
// 2.time: время задержки после отправки команды (default=0). Обычно >0 при отправке инструкций;  
//=================================================================================================
bool  deviceSend(uint8_t num, uint8_t numChip, uint8_t val, char code, uint16_t time=0){
//-------------------------------------------------------------------------------------------------
if(!checkNum(num)) {return 1;} //--Проверка на наличие активного ID(для защиты от фатальных сбоев);
//-------------------------------------------------------------------------------------------------
if(ptr[num]->stepSend==0){		
if(ptr[num]->mode=='I') {ptr[num]->codeErr=deviceSendi2c(num,numChip,val,code);}  
if(ptr[num]->mode=='S') {ptr[num]->codeErr=deviceSendSPI(num,numChip,val,code);} 
if(ptr[num]->mode=='s') {ptr[num]->codeErr=deviceSendSSPI(num,numChip,val,code);}
if(ptr[num]->mode=='4') {ptr[num]->codeErr=deviceSendBus4(num,numChip,val,code);}
if(ptr[num]->mode=='8') {ptr[num]->codeErr=deviceSendBus8(num,numChip,val,code);}
//-------------------------------------------------------------------------------------------------
if(time==0){return 1;} 
ptr[num]->timeDelay=micros(); ptr[num]->stepSend=80; return 0;}
//-------------------------------------------------------------------------------------------------
if(ptr[num]->stepSend==80){if(!(RT_HW_Base.getPastMcs(ptr[num]->timeDelay)>=time))  {return 0;} ptr[num]->stepSend=0; return 1;}	
//-------------------------------------------------------------------------------------------------
return 1;}
//=================================================================================================
//				Подготовка массива для отправки в Lcd
//=================================================================================================
void deviceTypeOne(uint8_t num, uint8_t numChip, uint8_t val, char code, uint8_t *arr){
uint8_t buff, bitE; 									//--Временные переменные;
//-------------------------------------------------------------------------------------------------
if(code=='D'){ptr[num]->cmnd=1;} else{ptr[num]->cmnd=0;}//--Запись флага cmnd;
if(numChip==1){bitE=RT_HW_HD44780_BIT_E;} 				//--Выбор номера бита E для Lcd c одним chip;
else 		  {bitE=RT_HW_HD44780_BIT_RW;}				//--Выбор номера бита E для Lcd c двумя chips;
buff=(val & 0xf0); 										//--Подготовка битов 7-4 для записи в Lcd;	
//-------------------------------------------------------------------------------------------------
if(code!='T'){
bitWrite(buff,RT_HW_HD44780_BIT_RS, ptr[num]->cmnd);	//--Установка бита команды (RS Lcd);
bitWrite(buff,RT_HW_HD44780_BIT_ON, ptr[num]->extLight);//--Установка пина подсветки;	
bitSet  (buff,bitE); arr[0] = buff; 					//--Запись байта с установленным битом E для соответствующего чипа;
bitClear(buff,bitE); arr[1] = buff;}	 				//--Запись байта с сброшенным    битом E для соответствующего чипа;
//-------------------------------------------------------------------------------------------------
buff=val<<4;											//--Подготовка битов 3-0 для записи в Lcd;	
bitWrite(buff,RT_HW_HD44780_BIT_RS, ptr[num]->cmnd);	//--Установка бита команды (RS Lcd);
bitWrite(buff,RT_HW_HD44780_BIT_ON, ptr[num]->extLight);//--Установка пина подсветки;	
bitSet  (buff,bitE); arr[2] = buff; 					//--Запись байта с установленным битом E для соответствующего чипа;
bitClear(buff,bitE); arr[3] = buff; 					//--Запись байта с сброшенным    битом E для соответствующего чипа;	
//-------------------------------------------------------------------------------------------------	
}
//================================================================================================= 
//				Отправка в i2c массива для записи в Lcd одного байта данных/инструкции
//=================================================================================================
uint8_t deviceSendi2c(uint8_t num, uint8_t numChip, uint8_t val, char code){
uint8_t arr[4]; deviceTypeOne(num,numChip,val,code,arr);
if(code!='T'){return RT_HW_Base.i2cSendBytes(&arr[0],4,ptr[num]->adr,ptr[num]->bus);}	
else         {return RT_HW_Base.i2cSendBytes(&arr[2],2,ptr[num]->adr,ptr[num]->bus);}}
//================================================================================================= 
//				Отправка в SPI массива для записи в Lcd одного байта данных/инструкции
//=================================================================================================
uint8_t deviceSendSPI(uint8_t num, uint8_t numChip, uint8_t val, char code){
uint8_t arr[4]; deviceTypeOne(num,numChip,val,code,arr);
RT_HW_PIN_DIR_ID idCS; RT_HW_PIN_DIR_SET_ID(idCS,ptr[num]->cs); RT_HW_PIN_DIR_WRITE_HIGH(idCS); RT_HW_PIN_DIR_MODE_OUTPUT(idCS);
uint8_t bus=ptr[num]->bus;
if(bus==8){RT_HW_Base.spiSetPinSCK(ptr[num]->sck,8); RT_HW_Base.spiSetPinMOSI(ptr[num]->mosi,8);}
if(code!='T')  {RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[0],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS); 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[1],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS);} 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[2],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS); 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[3],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS);	
return 0;}
//================================================================================================= 
//				Отправка в SSPI массива для записи в Lcd одного байта данных/инструкции
//=================================================================================================
uint8_t deviceSendSSPI(uint8_t num, uint8_t numChip, uint8_t val, char code){
uint8_t arr[4]; deviceTypeOne(num,numChip,val,code,arr);
RT_HW_PIN_DIR_ID idCS; RT_HW_PIN_DIR_SET_ID(idCS,ptr[num]->cs); RT_HW_PIN_DIR_WRITE_HIGH(idCS); RT_HW_PIN_DIR_MODE_OUTPUT(idCS);
uint8_t bus=ptr[num]->bus;
if(bus==8){RT_HW_Base.spiSetPinSCK(ptr[num]->sck,8); RT_HW_Base.spiSetPinMOSI(ptr[num]->mosi,8);}
if(code!='T')  {RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[0],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS); 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[1],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS);} 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[2],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS); 
				RT_HW_PIN_DIR_WRITE_LOW(idCS); RT_HW_Base.spiWrite(arr[3],bus);	RT_HW_PIN_DIR_WRITE_HIGH(idCS);	
return 0;}
				  
//================================================================================================= 
//				Отправка в BUS4 массива для записи в Lcd одного байта данных/инструкции
//=================================================================================================
uint8_t deviceSendBus4(uint8_t num, uint8_t numChip, uint8_t val, char code){
RT_HW_PIN_DIR_ID idPin; uint8_t pinEN; 
if(numChip==1){pinEN=ptr[num]->pin[2];} else {pinEN=ptr[num]->pin[1];} if(pinEN==255){return 0;} 
uint8_t cmnd=0; if(code=='D') {cmnd=1;} 
						  RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[0]);    RT_HW_PIN_DIR_WRITE(idPin,cmnd); //--Запись на пин RS;
if(ptr[num]->pin[1]!=255){RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[1]);    RT_HW_PIN_DIR_WRITE_LOW(idPin);} //--Сброс пина RW;
					      RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[2]);    RT_HW_PIN_DIR_WRITE_LOW(idPin);  //--Сброс пина E;
if(code!='T'){
for(uint8_t i=0; i<4;i++){RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[7+i]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,(4+i)));}		
                          RT_HW_PIN_DIR_SET_ID(idPin, pinEN);              RT_HW_PIN_DIR_WRITE_HIGH(idPin);  RT_HW_PIN_DIR_WRITE_LOW(idPin);}	//--Строб записи пином pinEN;
for(uint8_t i=0; i<4;i++){RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[7+i]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,(i)));}
                          RT_HW_PIN_DIR_SET_ID(idPin, pinEN);              RT_HW_PIN_DIR_WRITE_HIGH(idPin);  RT_HW_PIN_DIR_WRITE_LOW(idPin);	 //--Строб записи пином pinEN;												
//-------------------------------------------------------------------------------------------------
return 0;}	
//================================================================================================= 
//				Отправка в BUS8 массива для записи в Lcd одного байта данных/инструкции
//=================================================================================================
uint8_t deviceSendBus8(uint8_t num, uint8_t numChip, uint8_t val, char code){
RT_HW_PIN_DIR_ID idPin; uint8_t pinEN; 
if(numChip==1){pinEN=ptr[num]->pin[2];}   else{pinEN=ptr[num]->pin[1];} if(pinEN==255){return 0;} 
uint8_t cmnd=0; if(code=='D') {cmnd=1;} 
						  RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[0]);   RT_HW_PIN_DIR_WRITE(idPin,cmnd); //--Запись на пин RS;
if(ptr[num]->pin[1]!=255){RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[1]);   RT_HW_PIN_DIR_WRITE_LOW(idPin);} //--Сброс пина RW;
					      RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[2]);   RT_HW_PIN_DIR_WRITE_LOW(idPin);  //--Сброс пина E;
for(uint8_t i=0;i<8;i++) {RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[3+i]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,i));}//--Запись на пины D0-D7;
                          RT_HW_PIN_DIR_SET_ID(idPin,pinEN);              RT_HW_PIN_DIR_WRITE_HIGH(idPin);  RT_HW_PIN_DIR_WRITE_LOW(idPin);	//--Строб записи пином pinEN;
//-------------------------------------------------------------------------------------------------
return 0;}






























//RT_HW_Base.consoleVar(numChip,','); RT_HW_Base.consoleVar(val,',',0,'H'); 
//RT_HW_Base.consoleVar(code,',');    RT_HW_Base.consoleVar(time,'E');

//if(code=='D') {ptr[num]->cmnd=1;} else {ptr[num]->cmnd=0;}
//if((ptr[num]->mode=='I') || (ptr[num]->mode=='i') || (ptr[num]->mode=='S') || (ptr[num]->mode=='s')){ptr[num]->stepSend=10;} 
//if( ptr[num]->mode=='4') 																			{ptr[num]->stepSend=30;} 
//if( ptr[num]->mode=='8') 																			{ptr[num]->stepSend=40;} 
//if( ptr[num]->stepSend==0) 	{return 1;}
//}
//=================================================================================================
//                		 Шины i2c(PCA6587), SPI,SSPI(74HC595)
//=================================================================================================
//if(ptr[num]->stepSend==10) {		
//------Начало подготовки данных для инструкции/команды--------------------------------------------
//uint8_t arr[4]; deviceSend_createRegDirect(num,numChip,val,code,arr);
/*
uint8_t buff; 
uint8_t bitE; 
	 if(numChip==1){bitE=RT_HW_HD44780_BIT_E;} 
	 else          {bitE=RT_HW_HD44780_BIT_RW;}	//--Выбор номера бита управления E;
//else {return 7;}
//-------------------------------------------------------------------------------------------------
//									Формирование массива данных
//-------------------------------------------------------------------------------------------------
buff=(val & 0xf0); 										//--Подготовка битов 7-4 для записи в Lcd;	
bitWrite(buff,RT_HW_HD44780_BIT_RS, ptr[num]->cmnd);	//--Установка бита команды (RS Lcd);
bitWrite(buff,RT_HW_HD44780_BIT_ON, ptr[num]->extLed);	//--Установка пина подсветки;	
bitSet  (buff,bitE); arr[0] = buff; 					//--Запись байта с установленным битом E для соответствующего чипа;
bitClear(buff,bitE); arr[1] = buff;	 					//--Запись байта с сброшенным    битом E для соответствующего чипа;
buff=val<<4;											//--Подготовка битов 3-0 для записи в Lcd;	
bitWrite(buff,RT_HW_HD44780_BIT_RS, ptr[num]->cmnd);	//--Установка бита команды (RS Lcd);
bitWrite(buff,RT_HW_HD44780_BIT_ON, ptr[num]->extLed);	//--Установка пина подсветки;	
bitSet  (buff,bitE); arr[2] = buff; 					//--Запись байта с установленным битом E для соответствующего чипа;
bitClear(buff,bitE); arr[3] = buff;	 					//--Запись байта с сброшенным    битом E для соответствующего чипа;
*/

//-------------------------------------------------------------------------------------------------
//									Отправка в i2c
//-------------------------------------------------------------------------------------------------
//uint8_t err=0;
//if(ptr[num]->mode=='I'){
//RT_HW_Base.consoleVar(ptr[num]->pin[0],','); 
//RT_HW_Base.consoleVar(ptr[num]->pin[1],'_'); 
//RT_HW_Base.consoleArrVar(arr,'E',0,'H',4);	
//uint8_t err=deviceSend_transfer_i2c(num,code,arr); if(err) {ptr[num]->stepSend=0; return err;} else {ptr[num]->stepSend=80;}}
//}
	
	
//if(code!='T'){
//err=RT_HW_Base.WireSendBytes(ptr[num]->pin[0],&arr[0],4,ptr[num]->pin[1]); if(err!=0) {ptr[num]->stepSend=0; return err; }}	
//-------------------------------------------------------------------------------------------------
//else{
//err=RT_HW_Base.WireSendBytes(ptr[num]->pin[0],&arr[2],2,ptr[num]->pin[1]); if(err!=0) {ptr[num]->stepSend=0; return err; }}}


//ptr[num]->stepSend=80;}
/*
//-------------------------------------------------------------------------------------------------
//									Отправка в SPI
//-------------------------------------------------------------------------------------------------
if(ptr[num]->mode=='S'){
if(code!='T'){
RT_HW_Base.spiSendByte(ptr[num]->pin[0],ptr[num]->pin[1], arr[0]);
RT_HW_Base.spiSendByte(ptr[num]->pin[0],ptr[num]->pin[1], arr[1]);}			   
//----------------------------------------------------------------
RT_HW_Base.spiSendByte(ptr[num]->pin[0],ptr[num]->pin[1], arr[2]);
RT_HW_Base.spiSendByte(ptr[num]->pin[0],ptr[num]->pin[1], arr[3]);}	
//-------------------------------------------------------------------------------------------------
//									Отправка в SSPI
//-------------------------------------------------------------------------------------------------
if(ptr[num]->mode=='s'){ 
if(code!='T'){
RT_HW_Base.sspiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],ptr[num]->pin[2], arr[0]);
RT_HW_Base.sspiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],ptr[num]->pin[2], arr[1]);}
//-----------------------------------------------------------------------------------
RT_HW_Base.sspiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],ptr[num]->pin[2], arr[2]);
RT_HW_Base.sspiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],ptr[num]->pin[2], arr[3]);}	  
//-------------------------------------------------------------------------------------------------
ptr[num]->stepSend=80;} //--if(ptr[num]->stepSend==10){
//=================================================================================================
//                		 			  Шина BUS4
//			arr[]>RS[0], E1[1], DB7[2], DB6[3], DB5[4], DB4[5] ,E2[6], Led[7]; 
//=================================================================================================
if(ptr[num]->stepSend==30) {		
//-------------------------------------------------------------------------------------------------
//							Начало подготовки данных для инструкции/команды
//-------------------------------------------------------------------------------------------------
RT_HW_PIN_DIR_ID idPin;
//-------------------------------------------------------------------------------------------------
//								Запись LOW->pin RW,pin E; Установка пинов Led,RS
//-------------------------------------------------------------------------------------------------
						   RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[1]);   
						   RT_HW_PIN_DIR_WRITE_LOW(idPin);					//--Сброс пина E1;
if(ptr[num]->pin[6]!=255) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[6]);   RT_HW_PIN_DIR_WRITE_LOW(idPin);}				//--Сброс пина E2 (если пин предусмотрен);
if(ptr[num]->pin[7]!=255) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[7]);   RT_HW_PIN_DIR_WRITE(idPin,ptr[num]->extLed);} 	//--Установка подсветки дисплея (если пин предусмотрен);
						   RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[0]);   RT_HW_PIN_DIR_WRITE(idPin,ptr[num]->cmnd); 		//--Установка RS (данные/команда);	
//-------------------------------------------------------------------------------------------------
//								Запись в пины DB7-DB4
//-------------------------------------------------------------------------------------------------
if(code!='T'){
for(uint8_t i=0; i<4;i++) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[i+2]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,(7-i)));}
if(ptr[num]->chip==1) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[1]);} 
if(ptr[num]->chip==2) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[6]);}		
RT_HW_PIN_DIR_WRITE_HIGH(idPin);	RT_HW_PIN_DIR_WRITE_LOW(idPin);					//--Строб пина E1(E2);
}													
//-------------------------------------------------------------------------------------------------
//								Запись в пины DB3-DB0
//-------------------------------------------------------------------------------------------------
for(uint8_t i=0; i<4;i++) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[i+2]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,(3-i)));}
if(ptr[num]->chip==1) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[1]);} 
if(ptr[num]->chip==2) {RT_HW_PIN_DIR_SET_ID(idPin, ptr[num]->pin[6]);}		

RT_HW_PIN_DIR_WRITE_HIGH(idPin);	RT_HW_PIN_DIR_WRITE_LOW(idPin);					//--Строб пина E1(E2);	
//-------------------------------------------------------------------------------------------------
//									Завершение
//-------------------------------------------------------------------------------------------------
ptr[num]->stepSend=80;}	//--END if(ptr[num]->stepSend==30) {	
//=================================================================================================
//									Шина BUS8
//=================================================================================================
if(ptr[num]->stepSend==40) {	
RT_HW_Base.consoleVar(val,'E',0,'H');
//-------------------------------------------------------------------------------------------------
//							Начало подготовки данных для инструкции/команды
//-------------------------------------------------------------------------------------------------
RT_HW_PIN_DIR_ID idPin; uint8_t pinEN; uint8_t pinRS=ptr[num]->pin[0]; uint8_t pinRW=ptr[num]->pin[1]; uint8_t pinE =ptr[num]->pin[2];
//-------------------------------------------------------------------------------------------------
//							Подготовка пинов RS,RW,E
//-------------------------------------------------------------------------------------------------
if(numChip==1){pinEN=pinE;  if(pinRW!=255) {RT_HW_PIN_DIR_SET_ID(idPin,pinRW); RT_HW_PIN_DIR_WRITE_HIGH(idPin);}} //--Если номер чипа 1, pinRW=1;
else		  {pinEN=pinRW;                 RT_HW_PIN_DIR_SET_ID(idPin,pinE);  RT_HW_PIN_DIR_WRITE_LOW(idPin);}
if(pinE==255) {ptr[num]->codeErr= 90; return 0;} 	//--Выход по ошибке (предварительный контроль также осуществлен при инициализации);
//-------------------------------------------------------------------------------------------------
//							Запись на пин RS
//-------------------------------------------------------------------------------------------------
RT_HW_PIN_DIR_SET_ID(idPin,pinRS); RT_HW_PIN_DIR_WRITE(idPin,ptr[num]->cmnd);	
//-------------------------------------------------------------------------------------------------
//							Запись на пины D0-D7
//-------------------------------------------------------------------------------------------------
for(uint8_t i=0;i<8;i++) {RT_HW_PIN_DIR_SET_ID(idPin,ptr[num]->pin[3+i]); RT_HW_PIN_DIR_WRITE(idPin,bitRead(val,i));}
//-------------------------------------------------------------------------------------------------
//							Строб записи пином pinEN
//-------------------------------------------------------------------------------------------------
RT_HW_PIN_DIR_SET_ID(idPin,pinEN); RT_HW_PIN_DIR_WRITE_HIGH(idPin); RT_HW_PIN_DIR_WRITE_LOW(idPin);
//-------------------------------------------------------------------------------------------------
//									Завершение
//-------------------------------------------------------------------------------------------------
ptr[num]->stepSend=80;} //--END if(ptr[num]->stepSend==40) {
//=================================================================================================
*/




/*
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				Инициализация i2c arr[2]->cs,bus;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t initBus_i2c(uint8_t num){	
if(ptr[num]->pin[0]>=254) {return 50;}													//--Выход, если не допустимый адрес устройства i2c;
RT_HW_Base.WireBegin(ptr[num]->pin[1]); 												//--Настройка   шины i2cX как Master;
if(!RT_HW_Base.WireGetBitState(RT_HW_I2C_STATE_MASTER,ptr[num]->pin[1])) {return 51;}	//--Выход, если шина i2cX не настроена; 
return 0;}	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				Инициализация SPI arr[2]->cs,bus;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t initBus_SPI(uint8_t num){
if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,ptr[num]->pin[0])) {return 55;}			//--Выход, если не допустимый номер пина cs устройства SPI;																
digitalWrite(ptr[num]->pin[0],1); pinMode(ptr[num]->pin[0],OUTPUT);						//--Настройка пина CS на выход с установкой в HIGH;
RT_HW_Base.spiBegin(255,255,255,0,ptr[num]->pin[1]);									//--Настройка SPIx;
if(!RT_HW_Base.spiCheckBusState(RT_HW_SPI_STATE_MASTER,ptr[num]->pin[1])){return 56;}	//--Выход, если шина SPIx не настроена;	
RT_HW_Base.spiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],0);							//--Отправка 0 в SPIx;
return 0;}							
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				Инициализация SSPI arr[3]->clk,mosi,cs;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t initBus_SSPI(uint8_t num){
uint8_t buff=60;
for(uint8_t i=0;i<3;i++){
 if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,ptr[num]->pin[i])) {return (buff+i);}	//--Выход, если не допустимый номер пина SSPI;																
 else{digitalWrite(ptr[num]->pin[i],LOW); pinMode(ptr[num]->pin[i],OUTPUT);}}			//--Настройка пинов на дискретный вывод с установкой 0;
digitalWrite(ptr[num]->pin[2],HIGH); 													//--Установка пина cs;
RT_HW_Base.sspiSendByte(ptr[num]->pin[0],ptr[num]->pin[1],ptr[num]->pin[2],0);			//--Отправка 0 в SSPI;
return 0;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				Инициализация BUS8 arr[7]->RS,E1,DB7,DB6,DB5,DB4,E2,
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t initBus_BUSx(uint8_t num){
uint8_t buff=70; uint8_t qnt=0; uint8_t pin;
if(ptr[num]->mode=='4') {if(ptr[num]->chip==1) {qnt= 6;} else {qnt= 7;}}
if(ptr[num]->mode=='8') {if(ptr[num]->chip==1) {qnt=10;} else {qnt=11;}}	
if(qnt==0) {return buff;}													//--Выход без настройки err=70, если неверный код шины;
buff++;																		//--Установка начального кода ошибки;
for(uint8_t i=0;i<qnt;i++){													//--Перебор используемых пинов;
 pin=ptr[num]->pin[i];														//--Сохранение обрабатываемого пина;
 if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,pin)) {return (buff+i);}	//--Выход, если недоступный пин; 
 digitalWrite(pin,0); pinMode(pin,OUTPUT);}									//--Настройка пина на выход с установленным 0;
return 0;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				Инициализация шины
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t initBus(uint8_t num){
if(ptr[num]->mode=='I') {return initBus_i2c(num);}	
if(ptr[num]->mode=='S') {return initBus_SPI(num);}
if(ptr[num]->mode=='s') {return initBus_SSPI(num);}	
if((ptr[num]->mode=='4') || (ptr[num]->mode=='8')) {return initBus_BUSx(num);}	
return 99;}
*/

/*
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			Инициализация дисплев через 4-х битную шину данных (кроме дисплеев 40x4)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool initBUS4(RT_HW_HD44780_DEVICE_ID &z){
if(!deviceSend(z.num, pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3)),
				(char)pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+1)), 
				      pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+2))*100)) {return 0;}
			   if(++z.index<13){return 0;} 
z.index=0; return 1;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			Инициализация дисплев через 8-х битную шину данных (кроме дисплеев 40x4)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool initBUS8(RT_HW_HD44780_DEVICE_ID &z){
if(!deviceSend(z.num, pgm_read_byte(RT_HW_HD44780_INIT_BUS8+(z.index*3)),
		    (char)pgm_read_byte(RT_HW_HD44780_INIT_BUS8+(z.index*3+1)),
			      pgm_read_byte(RT_HW_HD44780_INIT_BUS8+(z.index*3+2))*100)) {return 0;}
			if(++z.index<13){return 0;} 
z.index=0; return 1;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			Инициализация дисплев через 4-х битную шину данных для дисплеев 40x4
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool init4004(RT_HW_HD44780_DEVICE_ID &z){
if(z.step==0) {z.chip=1; z.index=0;  z.step++;}
if(z.step==1) {if(!deviceSend(z.num, pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3)),
				(char)pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+1)), 
				      pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+2))*100)) {return 0;}
			   if(++z.index<13){return 0;} 
			   z.step++;}
if(z.step==2) {if((z.cols==40) && (z.rows==4)) {z.chip=2;  z.timeDelay=millis(); z.index=0; z.step++; return 0;} else {z.step=5; return 0;}}
if(z.step==3) {if(RT_HW_Base.getPastMs(z.timeDelay) >= 1100) {z.step++;} return 0;}
if(z.step==4) {if(!deviceSend(z.num, pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3)),
				(char)pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+1)), pgm_read_byte(RT_HW_HD44780_INIT_BUS4+(z.index*3+2))*100)) {return 0;}
			   if(++z.index<13){return 0;} 
			   z.timeDelay=millis(); z.step++;}
if(z.step==5) {if(RT_HW_Base.getPastMs(z.timeDelay) >= 100) {z.step++;} return 0;}
if(z.step==6) {z.chip=1; z.step=0; return 1;}	
return 1;};
//=================================================================================================	
*/		   

//-------------------------------------------------------------------------------------------------
//				Инициализация i2c arr[2]->adr,bus;
//-------------------------------------------------------------------------------------------------
//if(z->mode=='I'){
//if(z->pin[0]>=254) {return 50;}													//--Выход, если не допустимый адрес устройства i2c;
//RT_HW_Base.WireBegin(z->pin[1]); 												//--Настройка   шины i2cX как Master;
//if(!RT_HW_Base.WireGetBitState(RT_HW_I2C_STATE_MASTER,z->pin[1])) {return 51;}	//--Выход, если шина i2cX не настроена; 
//return 0;}																		//--Успешный выход;
//-------------------------------------------------------------------------------------------------
//				Инициализация SPI arr[2]->cs,bus;
//-------------------------------------------------------------------------------------------------
//if(z->mode=='S'){
//if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID, z->pin[0])) {return 55;}		//--Выход, если не допустимый номер пина cs устройства SPI;																
//digitalWrite(z->pin[0],1); pinMode(z->pin[0],OUTPUT);							//--Настройка пина CS на выход с установкой в HIGH;
//RT_HW_Base.spiBegin(255,255,255,0,z->pin[1]);									//--Настройка SPIx;
//if(!RT_HW_Base.spiCheckBusState(RT_HW_SPI_STATE_MASTER,z->pin[1])){return 56;}	//--Выход, если шина SPIx не настроена;	
//RT_HW_Base.spiSendByte(z->pin[0],z->pin[1],0);									//--Отправка 0 в SPIx;
//return 0;}							
//-------------------------------------------------------------------------------------------------
//				Инициализация SSPI arr[3]->clk,mosi,cs;
//-------------------------------------------------------------------------------------------------
//if(z.mode=='s'){
//uint8_t buff=60;
//for(uint8_t i=0;i<3;i++){
// if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z.pin[i])) {return (buff+i);}	//--Выход, если не допустимый номер пина SSPI;																
// else{digitalWrite(z.pin[i],LOW); pinMode(z.pin[i],OUTPUT);}}					//--Настройка пинов на дискретный вывод с установкой 0;
//digitalWrite(z.pin[2],HIGH); 													//--Установка пина cs;
//RT_HW_Base.sspiSendByte(z.pin[0],z.pin[1],z.pin[2],0);			//--Отправка 0 в SSPI;
//return 0;}

//-------------------------------------------------------------------------------------------------
//				Инициализация BUS4 arr[7]->RS,E1,DB7,DB6,DB5,DB4,E2,LCD,
//-------------------------------------------------------------------------------------------------
//if(z->mode=='4'){
//uint8_t buff=70; uint8_t qnt=0; uint8_t pin;
//if(z.chip==1) {qnt=7;} else {qnt=8;}}											//--Выход без настройки err=70, если неверный код шины;																	//--Установка начального кода ошибки;
//for(uint8_t i=0;i<qnt;i++){														//--Перебор используемых пинов;
// if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[i])) {return (70+i);}	//--Выход, если недоступный пин; 
// digitalWrite(z->pin[i],0); pinMode(z->pin[i],OUTPUT);}									//--Настройка пина на выход с установленным 0;
//return 0;}
//-------------------------------------------------------------------------------------------------
//				Инициализация BUS8 arr[7]->RS,E1,DB7,DB6,DB5,DB4,E2,
//-------------------------------------------------------------------------------------------------
/*
if((z->mode=='4') || (z->mode=='8')){
if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[0])) {return (80);}		//--Выход, если RS недоступный пин; 
digitalWrite(z->pin[0],0); pinMode(z->pin[0],OUTPUT);							//  настройка пина на выход с установленным 0;
if(z->chip==2){																	//--Для LCD с 2-мя чипами
if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[1])) {return (81);}		//  выход, если RW недоступный пин (для LCD с 2-мя чипами);
digitalWrite(z->pin[1],0); pinMode(z->pin[1],OUTPUT);}							//  настройка пина на выход с установленным 0;	
if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[2])) {return (82);}		//--Выход, если E недоступный пин;
digitalWrite(z->pin[2],0); pinMode(z->pin[2],OUTPUT);							//--Настройка пина на выход с установленным 0;
uint8_t begin;
if(z->mode=='4') {begin=7;} else {begin=3;}
for(uint8_t i=begin; i<11; i++){												//--Перебор используемых пинов;
if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[i])) {return (80+i);}	//--Выход, если недоступный пин; 
digitalWrite(z->pin[i],0); pinMode(z->pin[i],OUTPUT);}							//--Настройка пина на выход с установленным 0;
return 0;}
//-------------------------------------------------------------------------------------------------
*/

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						Инициализация SSPI ->clk,mosi,cs
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//uint8_t initBUS_SSPI(RT_HW_HD44780_DEVICE_ID *z){
//for(uint8_t i=0;i<3;i++){
//if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,z->pin[i])) {return (55+i);}	//--Выход, если не допустимый номер пина SSPI;																
//else{digitalWrite(z->pin[i],LOW); pinMode(z->pin[i],OUTPUT);}}					//--Настройка пинов на дискретный вывод с установкой 0;
//digitalWrite(z->pin[2],HIGH); 													//--Установка пина cs;
//RT_HW_Base.sspiSendByte(z->pin[0],z->pin[1],z->pin[2],0);							//--Отправка 0 в SSPI;
//return 0;}