#include "RT_HW_BASE.h"	 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.Справочная игнформация
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ESP32->M5STACK:
// setRxTimeout устанавливает таймаут, по истечении которого будет вызван обратный вызов onReceive 
//  (после получения данных он ждет это время бездействия UART rx для вызова обратного вызова fnc)
// параметр symbol_timeout определяет порог тайм-аута в периодах символа uart. 
// Установка таймаута символа 0 отключает обратный вызов по таймауту.
// Настройка максимального тайм-аута рассчитывается IDF автоматически. 
// Если установлено выше максимального значения, оно игнорируется, 
//                                             и на Serial0 печатается ошибка (проверьте консоль).
// Примеры: максимум для 11-битного символа — 92 (SERIAL_8N2, SERIAL_8E1, SERIAL_8O1 и т. д.), 
//          максимум для 10-битного символа — 101 (SERIAL_8N1).
// Приме:   symbols_timeout=1 определяет тайм-аут, равный времени передачи одного символа (~11 бит) 
//                                                               на текущей скорости передачи данных.
//          Для скорости 9600 бод, SERIAL_8N1 (10-битный символ) и symbol_timeout = 3 
//                                                       тайм-аут будет 3 / (9600 / 10) = 3,125 мс;
//-------------------------------------------------------------------------------------------------
// ESP32->M5STACK:
// onReceive установит обратный вызов, который будет вызываться всякий раз, 
//              когда происходит прерывание UART (UART_INTR_RXFIFO_FULL или UART_INTR_RXFIFO_TOUT).
// Прерывание UART_INTR_RXFIFO_FULL срабатывает при получении UART_FULL_THRESH_DEFAULT байтов 
//                                                      (по умолчанию в IDF определено 120 байтов).
// Прерывание UART_INTR_RXFIFO_TOUT срабатывает при передаче символов UART_TOUT_THRESH_DEFAULT без приема 
//                                                      (по умолчанию в IDF определено 10 символов).
// Параметр onlyOnTimeout будет определять, как будет вести себя onReceive:
// По умолчанию: true -- Обратный вызов будет вызван только тогда, когда произойдет тайм-аут RX.
//                       Весь поток байтов будет сразу готов к чтению функцией обратного вызова.
//                       Эта опция может привести к переполнению Rx в зависимости от размера буфера Rx 
//                                                   и количества байтов, полученных в потоковой передаче.
//               false --Обратный вызов будет вызван, когда FIFO достигнет 120 байт, а также по тайм-ауту RX.
//                       Поток непереданных байтов будет "разбит" на блоки по 120 байт при каждом обратном вызове.
//                       Этот параметр позволяет избежать любого вида переполнения Rx, 
//                                       но оставляет работу по повторной сборке пакетов UART приложению.
//-------------------------------------------------------------------------------------------------
//void begin(unsigned long baud, uint32_t config=SERIAL_8N1, int8_t rxPin=-1, int8_t txPin=-1, bool invert=false, unsigned long timeout_ms = 20000UL, uint8_t rxfifo_full_thrhd = 112);
// uart_t* uartBegin(uint8_t uart_nr, uint32_t baudrate, uint32_t config, int8_t rxPin, int8_t txPin, uint16_t rx_buffer_size, uint16_t tx_buffer_size, bool inverted, uint8_t rxfifo_full_thrhd); //ESP32
// ESP32->M5STACK:
// uart_t* uartBegin(uint8_t uart_nr, uint32_t baudrate, uint32_t config, int8_t rxPin, int8_t txPin, uint16_t rx_buffer_size, uint16_t tx_buffer_size, bool inverted, uint8_t rxfifo_full_thrhd); //M5STACK
// setPins(int8_t rxPin, int8_t txPin, int8_t ctsPin = -1, int8_t rtsPin = -1) устанавливает пины UART.
//          										setPins() должна вызываться после Serial.begin().          
// setHwFlowCtrlMode(uint8_t mode = HW_FLOWCTRL_CTS_RTS, uint8_t threshold = 64) -
//                Включает или отключает аппаратное управление потоком с использованием выводов RTS и/или CTS 
//                                                          (перед этим необходимо использовать setPins()).

//-------------------------------------------------------------------------------------------------
// ESP32->ESP32S,ESP32S2,ESPS3,ESP32C3,M5STACK:
//extern HardwareSerial Serial0;
//#else
//extern HardwareSerial Serial;
//#endif
//
//#if SOC_UART_NUM > 1
//extern HardwareSerial Serial1;
//#endif
//#if SOC_UART_NUM > 2
//extern HardwareSerial Serial2;
//#endif
//=================================================================================================

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.СОЗДАНИЕ ДОПОЛНИТЕЛЬНЫХ ОБЪЕКТОВ UART ПОД ТИП MCU
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
#if defined(RT_HW_PERMIT_UARTS)
SoftwareSerial SerialS(RT_HW_UARTS_RX, RT_HW_UARTS_TX);
#endif

#if defined(RT_HW_PERMIT_UART1) && defined(RT_HW_CORE_STM32_D)
#if !(defined(RT_HW_BOARD_D_BLACK_F407VE) || defined(RT_HW_BOARD_D_BLACK_F407VG))
//HardwareSerial  Serial1(RT_HW_UART1_RX,RT_HW_UART1_TX);
#endif
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART2) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial2(RT_HW_UART2_RX,RT_HW_UART2_TX);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART3) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial3(RT_HW_UART3_RX,RT_HW_UART3_TX);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART4) && defined(RT_HW_CORE_STM32_D)
#if (defined(RT_HW_BOARD_D_BLACK_F407VE) || defined(RT_HW_BOARD_D_BLACK_F407VG))
HardwareSerial  Serial4(RT_HW_UART4_RX,RT_HW_UART4_TX);
#endif
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART5) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial5(RT_HW_UART5_RX,RT_HW_UART5_TX);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART6) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial6(RT_HW_UART6_RX,RT_HW_UART6_TX);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART7) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial7(RT_HW_UART7_RX,RT_HW_UART7_TX);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART8) && defined(RT_HW_CORE_STM32_D)
HardwareSerial  Serial8(RT_HW_UART8_RX,RT_HW_UART8_TX);
#endif
//-------------------------------------------------------------------------------------------------

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 		    2.Функции проверки флагов begin,readyслова состояния uart
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_BASE:: uartCheckBegin(uint8_t num){return bitRead(uart.begin,num);}
bool RT_HW_BASE:: uartCheckReady(uint8_t num){return bitRead(uart.ready,num);}
uint16_t RT_HW_BASE:: uartGetBegin(){return uart.begin;}
uint16_t RT_HW_BASE:: uartGetReady(){return uart.ready;}


//=================================================================================================
// 							3.Запись/чтение номеров пинов, скорости
//=================================================================================================
void    RT_HW_BASE:: uartSetState   (uint8_t state, uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.state=state;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.state=state;}
#endif
};
//-------------------------------------------------------------------------------------------------
void    RT_HW_BASE:: uartSetSpeed   (uint32_t speed, uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.speed=speed;}
#endif
};
//-------------------------------------------------------------------------------------------------
void    RT_HW_BASE:: uartSetConfig  (uint32_t config, uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.config=config;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.config=config;}
#endif
};
//-------------------------------------------------------------------------------------------------
uint8_t    RT_HW_BASE:: uartGetState   (uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.state;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.state;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.state;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.state;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.state;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.state;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.state;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.state;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.state;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.state;}
#endif
return 0;};
//-------------------------------------------------------------------------------------------------
uint32_t    RT_HW_BASE:: uartGetSpeed   (uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.speed;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.speed;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.speed;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.speed;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.speed;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.speed;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.speed;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.speed;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.speed;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.speed;}
#endif
return (uint32_t)115200;};
//-------------------------------------------------------------------------------------------------
uint32_t    RT_HW_BASE:: uartGetConfig   (uint8_t num){	
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.config;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.config;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.config;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.config;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.config;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.config;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.config;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.config;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.config;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.config;}
#endif
return 0x800001C;};
//-------------------------------------------------------------------------------------------------



//=================================================================================================
// 							3.Запись/чтение номеров пинов, скорости
//=================================================================================================
void    RT_HW_BASE:: uartSetPinRX   (uint8_t rx, uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.rx=rx;}
#endif
};
//-------------------------------------------------------------------------------------------------
void    RT_HW_BASE:: uartSetPinTX   (uint8_t tx, uint8_t num){	//--3.1.Запись номера пина SCL
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.tx=tx;}
#endif
};
//-------------------------------------------------------------------------------------------------
uint8_t RT_HW_BASE:: uartGetPinRX   (			 uint8_t num){	//--Чтение из регистра номера пинa RX;
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.rx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.rx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.rx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.rx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.rx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.rx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.rx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.rx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.rx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.rx;}
#endif
return 255;};
//-------------------------------------------------------------------------------------------------
uint8_t RT_HW_BASE:: uartGetPinTX   (			 uint8_t num){	//--Чтение из регистра номера пинa RX;
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.tx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.tx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.tx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.tx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.tx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.tx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.tx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.tx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.tx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.tx;}
#endif
return 255;};
//=================================================================================================




//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.1. Контроль наличия UART в контроллере по его номеру 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool RT_HW_BASE:: uartCheckNum(uint8_t num){
//-----------------
#if defined(RT_HW_PERMIT_UART0)
if(num==0){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART1)
if(num==1){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART2)
if(num==2){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART3)
if(num==3){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(num==4){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART5)
if(num==5){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(num==6){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(num==7){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(num==8){return 1;}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(num==9){return 1;}
#endif
return 0;};




//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.2. Функция uartBegin(n,speed)
//void begin(unsigned long baud, uint32_t config=SERIAL_8N1, int8_t rxPin=-1, int8_t txPin=-1, bool invert=false, unsigned long timeout_ms = 20000UL, uint8_t rxfifo_full_thrhd = 112);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void  RT_HW_BASE:: uartBegin (uint8_t n){
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART0)	
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==0){if(uartGetState(n)==0){uartSetState(1,n); Serial.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}                                                                                  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
if(n==0){if(uartGetState(n)==0){uartSetState(1,n); Serial.begin(uartGetSpeed(n));} return;}
#endif	
//-------------------------------------------------------------------------------------------------//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART1)
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==1){if(uartGetState(n)==0){uartSetState(1,n); Serial1.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
if(n==1){if(uartGetState(n)==0){uartSetState(1,n); Serial1.begin(uartGetSpeed(n));} return;}
#endif	
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART2)
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==2){if(uartGetState(n)==0){uartSetState(1,n); Serial2.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
if(n==2){if(uartGetState(n)==0){uartSetState(1,n); Serial2.begin(uartGetSpeed(n));} return;}
#endif	
//-------------------------------------------------------------------------------------------------	
#if	defined(RT_HW_PERMIT_UART3)
if(n==3){if(uartGetState(n)==0){uartSetState(1,n); Serial3.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){if(uartGetState(n)==0){uartSetState(1,n); Serial4.begin(uartGetSpeed(n));}}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){if(uartGetState(n)==0){uartSetState(1,n); Serial5.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){if(uartGetState(n)==0){uartSetState(1,n); Serial6.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){if(uartGetState(n)==0){uartSetState(1,n); Serial7.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){if(uartGetState(n)==0){uartSetState(1,n); Serial8.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){if(uartGetState(n)==0){uartSetState(1,n); SerialS.begin(uartGetSpeed(n));}}
#endif			
}	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.3. uartSerial(n) проверка подключения устройства.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool  RT_HW_BASE:: uartSerial (uint8_t n){
//--Проверка с учетом особенностей STM32 в среде STM32duino, SEEEDUINO
#ifdef RT_HW_PERMIT_UART0
if(n==0){
//--------Для программной среды STM32duino---------------------------------------------------------
#if defined(RT_HW_CORE_ESP32C3)
if(uartUSB.dir==4) {return 1;}
if(uartUSB.dir==0) {uartUSB.timeBegin=millis()+500;	 uartUSB.dir++;  return 0;}
if(uartUSB.dir==1) {if(millis()<(uartUSB.timeBegin))              	{return 0;} 				    
				    bitSet(uart.ready,0); 			 uartUSB.dir=4;  return 1;}
#endif
//--------Для программной среды STM32duino---------------------------------------------------------
#if defined(RT_HW_CORE_STM32_D) && defined (USBCON) && defined(USBD_USE_CDC) 
if(uartUSB.dir==4) {return 1;}
if(uartUSB.dir==0) {uartUSB.timeBegin=millis()+500;	 uartUSB.dir++;  return 0;}
if(uartUSB.dir==1) {if(millis()<(uartUSB.timeBegin))              	{return 0;} 				    
					if(CDC_connected()){
					uartUSB.timeBegin=millis()+20; 	 uartUSB.dir++;	 return 1;}
                    uartUSB.timeBegin=millis()+500;               	 return 0;}
if(uartUSB.dir==2) {if(millis()<uartUSB.timeBegin)                  {return 0;}
				    bitSet(uart.ready,0); 			 uartUSB.dir=4;  return 1;} 
//--------Для программной среды Seeeduino---------------------------------------------------------
#elif defined(RT_HW_CORE_PI_PICO)
if(uartUSB.dir==4) {												 return 1;}	//--Пустой цикл по готовности USB;
if(uartUSB.dir==0) {uartUSB.timeBegin=millis();   	 uartUSB.dir++;  return 0;}	//--Подготовка таймера;
if(uartUSB.dir==1) {if((millis()-uartUSB.timeBegin)<1000) 	      	{return 0;}	//--Пауза 1000мс; 
				    uartUSB.timeBegin=millis();									//--Подготовка таймера;
					if(!Serial)									  	{return 0;}	//--Проверка готовности USB;
					uartUSB.dir++;									 return 0;} //--Переход на технологическую паузу;
if(uartUSB.dir==2) {if((millis()-uartUSB.timeBegin)<20) 	      	{return 0;} //--Пауза 20мс;
                    bitSet(uart.ready,0);           uartUSB.dir=4;   return 0;} //--Установка флага готовности и переход на завершение;
//if(uartUSB.dir==2) { {uartUSB.dir=0; return 0;} 
//                    bitSet(uart.ready,0);           uartUSB.dir=4; return 1;} 
//--------Для программной среды Seeeduino---------------------------------------------------------
//#elif defined(RT_HW_CORE_SEEED)
//if(device.uart.dir0==10){return 1;}
//if(device.uart.dir0==0) {             device.uart.timeStart=millis();                                 device.uart.dir0++;  return 0;}
//if(device.uart.dir0==1) {if(getPastMs(device.uart.timeStart)>1500)   {device.uart.timeStart=millis(); device.uart.dir0++;} return 0;}
//if(device.uart.dir0==2) {if(getPastMs(device.uart.timeStart)> 500)                                   {device.uart.dir0++;} return 0;}
//if(device.uart.dir0==3)	{device.uart.timeStart=millis(); 
						  //if(!Serial.dtr()){device.uart.dir0=2;} 						 
//						 if(!Serial){device.uart.dir0=2;} 
//						 else {device.uart.dir0++;} return 0;}
//if(device.uart.dir0==4) {if(getPastMs(device.uart.timeStart) >20) {bitSet(device.uart.ready,0); device.uart.dir0=10;}      return 0;}			 
#else	
//--------Для других программных среды (avr,esp8266,esp32,sam,STM32master_F1,STM32master_F4------	
if(Serial) {bitSet(uart.ready,n); return 1;} else {return 0;}
#endif
//-----------------------------------------------------------------------------------------------
}
#endif
//------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_UART1)
if(n==1){if(Serial1){bitSet(uart.ready,n); return 1;}}
#endif		
#if defined(RT_HW_PERMIT_UART2)
if(n==2){if(Serial2){bitSet(uart.ready,n); return 1;}}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){if(Serial3){bitSet(uart.ready,n); return 1;}}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){if(Serial4){bitSet(uart.ready,n); return 1;}}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){if(Serial5){bitSet(uart.ready,n); return 1;}}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){if(Serial6){bitSet(uart.ready,n); return 1;}}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){if(Serial7){bitSet(uart.ready,n); return 1;}}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){if(Serial8){bitSet(uart.ready,n); return 1;}}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){if(SerialS){bitSet(uart.ready,n); return 1;}}
#endif	
return 0;	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.4. uartAvaible(n)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t  RT_HW_BASE:: uartAvailable(uint8_t n){
#if defined(RT_HW_PERMIT_UART0)
if(n==0){return Serial.available();}
#endif	
#if defined(RT_HW_PERMIT_UART1) && !defined(RT_HW_BLOCK_UART1_READ) //---У ESP8266 Serail1 работает только на выход;
if(n==1){return Serial1.available();}
#endif		
#if defined(RT_HW_PERMIT_UART2)
if(n==2){return Serial2.available();}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){return Serial3.available();}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){return Serial4.available();}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){return Serial5.available();}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){return Serial6.available();}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){return Serial7.available();}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){return Serial8.available();}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){return SerialS.available();}
#endif	
return 	0;	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.5. uartAvaibleForWrite(n)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t  RT_HW_BASE:: uartAvailableForWrite(uint8_t n){
#if !defined(RT_HW_BLOCK_AVAILABLE_FOR_WRITE) 	//--Не везде присутствует функция Serial.availableForWrite();													
#if defined(RT_HW_PERMIT_UART0)
if(n==0){return Serial.availableForWrite();}
#endif	
#if defined(RT_HW_PERMIT_UART1)
if(n==1){return Serial1.availableForWrite();}
#endif		
#if defined(RT_HW_PERMIT_UART2)
if(n==2){return Serial2.availableForWrite();}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){return Serial3.availableForWrite();}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){return Serial4.availableForWrite();}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){return Serial5.availableForWrite();}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){return Serial6.availableForWrite();}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){return Serial7.availableForWrite();}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){return Serial8.availableForWrite();}
#endif
#if defined(RT_HW_PERMIT_UARTS)
//if(n==9){return SerialS.availableForWrite();}
#endif
//---------------------------------------------------------
RT_HW_Base.buff.vu8=n;	//--Для устранения предупреждений от C++;
return 63;	
#endif		//--#if !defined(RT_HW_BLOCK_AVAILABLE_FOR_WRITE) 
return 	63;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.6. uartRead(n)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t RT_HW_BASE:: uartRead(uint8_t n){
#if defined(RT_HW_PERMIT_UART0)
if(n==0){return Serial.read();}
#endif	
#if defined(RT_HW_PERMIT_UART1) && !defined(RT_HW_BLOCK_UART1_READ) //---У ESP8266 Serail1 работает только на выход;
if(n==1){return Serial1.read();}
#endif		
#if defined(RT_HW_PERMIT_UART2) 
if(n==2){return Serial2.read();}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){return Serial3.read();}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){return Serial4.read();}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){return Serial5.read();}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){return Serial6.read();}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){return Serial7.read();}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){return Serial8.read();}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){return SerialS.read();}
#endif	
return 	0;	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.7. uartWrite(n,val)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_BASE:: uartWrite(uint8_t n, uint8_t val){
#if defined(RT_HW_PERMIT_UART0)
if(n==0){
#if defined(RT_HW_CORE_PI_PICO)
	Serial.print(val);
#else
	Serial.write(val);
#endif	
}
#endif	
#if defined(RT_HW_PERMIT_UART1)
if(n==1){Serial1.write(val);}
#endif		
#if defined(RT_HW_PERMIT_UART2)
if(n==2){Serial2.write(val);}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){Serial3.write(val);}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){Serial4.write(val);}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){Serial5.write(val);}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){Serial6.write(val);}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){Serial7.write(val);}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){Serial8.write(val);}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){SerialS.write(val);}
#endif		
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			14.78 uartPrint(n,str)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_BASE:: uartPrint(uint8_t n, String str){
#if defined(RT_HW_PERMIT_UART0)
if(n==0){Serial.print(str);}
#endif	
#if defined(RT_HW_PERMIT_UART1)
if(n==1){Serial1.print(str);}
#endif		
#if defined(RT_HW_PERMIT_UART2)
if(n==2){Serial2.print(str);}
#endif	
#if defined(RT_HW_PERMIT_UART3)
if(n==3){Serial3.print(str);}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){Serial4.print(str);}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){Serial5.print(str);}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){Serial6.print(str);}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){Serial7.print(str);}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){Serial8.print(str);}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){SerialS.print(str);}
#endif		
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	


//=================================================================================================
void RT_HW_BASE:: uartHeadInterface(uint8_t n){			//--Вывод заголовка по интерфейсу;										//--6.3.Вывод заголовка по интерфейсу;								
if(!uartCheckNum(n))			{return;}	//--Проверка наличия шины;
consoleBegin();	if(!console.ok)	{return;} 	//--Настройка консоли с параметрами по умолчанию и ожидание готовности;
mess.alig='R'; mess.blank='G'; mess.modePin='A'; mess.modeAdr='A'; mess.qntBit=8; mess.signBit='1'; 
//LinkUART0.rx=1; 
//uartSetPinRX(1,0); 
//uartSetPinTX(0,0); 
//uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,0),0);
//uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,1),0);
//uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,2),1);
//uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,3),1);
//uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,4),2);
//uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,5),2);
//consoleVar(String(F("__UART0(RX,TX):")));  consoleVar(LinkUART0.rx,','); consoleVar(LinkUART0.tx,'E');

uint8_t arr[]={uartGetPinRX(n),uartGetPinTX(n)}; 
consoleVar(String(F("UART"))); consoleVar(n,':'); 
consoleVar(String(F(" RX,TX="))); consoleArrVar(arr,';',0,'P',2); consoleBlank(15);
//uint8_t status=i2cGetStatus(bus);
//     if(status==1){consoleVar(String(F("  Master")),';');}
//else if(status==2){consoleVar(String(F("   Slave")),';');}
//else			  {consoleVar(String(F(" No init")),';');}
//consoleVar(String(F(" state=")));              		  consoleVar(i2cGetState(bus),':',0,'H');	 consoleVar(i2cGetState(bus),';',0,'B');
//consoleVar(String(F(" speed(kHz)="))); consoleVar((uint32_t)(i2cGetSpeed(bus)/1000),';');
consoleCR();
}
//=================================================================================================
//								Проверка готовности UART c настройкой (по небходимости) 
//=================================================================================================
/*
bool RT_HW_BASE::  UART_init(uint8_t num,uint32_t speed){	//==UART;
if(!bitRead(num,uart.begin)){uartBegin(num, speed); bitSet(uart.begin,num);}
if(!bitRead(num,uart.ready)){uartSerial(num);}
return bitRead(num,uart.ready);}
//=================================================================================================
//								Отправка байта в UART
//=================================================================================================
bool RT_HW_BASE:: UART_writeV8(uint8_t v, uint8_t num){
if(!uartInit){return 0;}
if(uartAvailableForWrite(num)>0){uartWrite(num,v); return;}
//-------------------------------------------------------------------------------------------------
uint32_t timeBegin=micros();
do{if(getPastMcs(timeBegin)>500) {return 0;} if(uartAvailableForWrite(num)>0){break;}}while(1);
uartWrite(num,v);
return 1;
};
*/