/**************************************************************************************************
 *       БИБЛИОТЕКА ДЛЯ РАБОТЫ С ДИСПЛЕЯМИ NEXTION
 *************************************************************************************************/
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Простые функции		
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Коды параметров
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ПЕРЕЧИСЛЕНИЯ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//enum{RT_HW_NEXT_PARAM_BLINK=0, RT_HW_NEXT_PARAM_NUM, RT_HW_NEXT_PARAM_UART, RT_HW_NEXT_PARAM_STATE};
//enum{RT_HW_NEXT_STATE_NOT_INIT=0, RT_HW_NEXT_STATE_BAN, RT_HW_NEXT_STATE_PERMIT, RT_HW_NEXT_STATE_READY, RT_HW_NEXT_STATE_BUSY};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Структура данных для каждого устройства - 
//			все данные хранятся в массиве RT_HW_NEXT_DEVICE_ID *ptr[RT_HW_NEXT_MAX_DEVICE+1],
//			создаваемом при создании класса RT_HW_NEXT.
//    Переводится из RT_HW_NEXT_STATE_READY в RT_HW_NEXT_STATE_BUSY специальной функцией, если число транзакций за один вход
//	  в задачу (для FLProg это плата) превышает заданное кол-во (например 3). Это делается для снижение вероятности переполнения
//	  кольцевого буферa UART и снижения временных задержек от задачи.
//	  В рабочем режиме при входе в задачу состояние всегда сбрасывается в RT_HW_NEXT_STATE_READY.  
//	num - уникальный номер устройства, назначаемый при инциализации (от 1 до RT_HW_NEXT_MAX_DEVICE(default=8).
//		  Если при инциализации выявляется недопустимая совокупность условий и параметров (недопустимый uart, все номера заняты и пр.),
//		  параметру присвативается значение =0;
//	uart,speed-номер и скорость настройки выбранного UART;
//  blink-битовое периодически(default=50mc) меняющий значение параметр;
//  Blink-битовое периодически(каждые 20 раз по счетчику Cnt(максиум=63) при смене blink->default=1000mc) меняющий значение параметр; 
//  event - событие по изменению blink;
//	Event - событие по изменени  Blink;
//  dir   - номер этапа для функции управления устройтством.
//  cntPack-Счетчик отправленных кол-во пакетов за одно вхождение в задачу и его ограничение;
//  cntNax -Ограничение кол-во пакетов, отправленных за одно вхождение в задачу;
//  err   - код ошибки (0-7) при работе с устройством
//  *** Примечание. Все параметры инициализируются конструктором.

//  Протокол передачи MCU->Nextion (все поля в текстовой форме):
//  "in"	- заголовок посылки;
//  "xx"    - номер регистра куда записывается значение. В Nextion эта переменная должна быть определена в разделе Program.s.
//				Пример: //--Переменные входные
//						int  in0,   in1,  in2, in3,  in4,   in5,  in6,  in7,  in8,  in9
//				В Nextion (v.1.63.3) используется только один формат переменных - int, что соответствует int32_t.
// "="		- знак равно
// "Целое число" - передаваемое число.

//  Протокол передачи Nextion->MCU:
//  0x3A	- (byte) идентификатор начала посылки;
//	func	- (byte) код функции: 	0x06, отправка регистров из byte массива;
//									0x16, отправка регистров из word массива;
//									0x26, отправка регистров из long массива;
//  adr		- (word) адрес первого  регистра передаваемого массива;
//  qnt		- (byte) кол-во передаваемых регистров;
//  xx,xx...- (byte,word,long) - массив передаваемых регистров;
//  crc 	- контрольная сумма;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


#ifndef RT_HW_NEXT_h
#define RT_HW_NEXT_h
//-------------------------------------------------------------------------------------------------
#include "RT_HW_BASE.h"						//--Подключение базовой библиотеки;
#define RT_HW_NEXT_VER			  11		//--Номер версии библиотеки;
#define RT_HW_NEXT_MAX_DEVICE	  8			//--Максимальнон кол-во допустимых устройств;
#define RT_HW_NEXT_PERIOD_BLINK   10 //50	//--Период тиков для тактового генератора;
#define RT_HW_NEXT_TIMEOUT_READ   100		//--Тайм-аут на принятие посылки;
#define RT_HW_NEXT_BUFF_REG_SIZE  40		//--Размер приемного буфера посылки из Nextion (8 байт служебные);
#define RT_HW_NEXT_WATCH_DOG_BUSY 10        //--
//#define RT_HW_NEXT_DEBUG					//--Флаг отладки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Структура данных для приема посылок из NEXTION по программному протоколу 
//			все данные хранятся в массиве RT_HW_NEXT_DEVICE_ID *ptr[RT_HW_NEXT_MAX_DEVICE+1],
//			создаваемом при создании класса RT_HW_NEXT.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_STRUCT_NEXT_GET_NEXTION {  		   
uint8_t  dir=0;						//--Управление
uint8_t  head,func;					//--Буферы: заголовок(0x3A), код функции(=0x06 send byte('B'), =0x16 send word('W'), =0x26 send long('L');
uint8_t  adr,qnt,crc,qntByte;				//--Буферы: адрес массива, кол-во регистров,контрольная сумма;
uint8_t  buff[RT_HW_NEXT_BUFF_REG_SIZE];	//--Буфер для приема данных      из принятой транзакции;
uint8_t  cnt;								//--Количество принятых байт;
uint8_t  codeErr=0;							//--Код ошибки;
uint32_t timeStart;
}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Структура данных для приема посылок из NEXTION по программному протоколу 
//			все данные хранятся в массиве RT_HW_NEXT_DEVICE_ID *ptr[RT_HW_NEXT_MAX_DEVICE+1],
//			создаваемом при создании класса RT_HW_NEXT.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_STRUCT_NEXT_DEVICE {  		   
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  num=0; 				//--Номер устройства (от 1 до MAX_DEVICE).По этому номеру внешние блоки привязываются к устройству;
uint8_t  custom=0;				//--Код настройки: =0 не настраивался; =1 успешно настроен; >1 код ошибки настройки; 
uint8_t  dir=0;					//--Этапы основной программы управления устройствами;
//-----------------Параметры корректируемые до вызова функции setID()------------------------------
uint8_t  uart=255;				//--Номер UART;
uint32_t speed=115200;			//--Скорость UART;

uint8_t  lenArrByte=2;			//--Длина массива uint8_t  arrByte[];
uint8_t  lenArrWord=1;			//--Длина массива uint16_t arrWord[];
uint8_t  lenArrLong=1;			//--Длина массива uint32_t arrLong[];
uint8_t	 *ptrArrByte;			//--Указатель на массив uint8_t  arrByte[];
uint16_t *ptrArrWord;			//--Указатель на массив uint16_t arrWord[];
uint32_t *ptrArrLong;			//--Указатель на массив uint32_t arrLong[];
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  ready=0;				//--Готовность устройства к обмену;
uint8_t  codeErr;				//--Код ошибки;
uint8_t  busy;
//----------------------------------------
uint32_t timeStart; 			//--Переменные для работы со временем генератора тактирования;
uint8_t  event:1, blink:1, eventReboot:1, blinkReboot:1, readyRead:1;	//--Флаги; 
uint8_t  durationByte=90;		//--Длительность передачи одного байта;
uint8_t  lenSendPack=12;		//--Длина посылки отправляемой транзакции;
uint16_t durationSendPack=1000;	//--
uint16_t durationGetPack=1000;
uint8_t  stepSendGen=0;
uint16_t delaySendGen;
uint32_t timeSendGen;
uint8_t  cntPack=0;				//--Счетчик отправленных пакетов;
//----------------------------------------
uint32_t timeGet;				//--Рабочий регистр;
uint8_t	 timeOutGetByte=10;		//--Тайм-аут на прием очередного байта транзакции;
uint8_t  index;					//--Счетчик кол-ва принятых байт;
uint8_t  step;
uint8_t  head,func,adr,qnt,crc;
uint32_t crcCalc;
uint16_t cntGetPackAll; 		//--Счетчик принятых всего    транзакций;
uint16_t cntGetPackOk;			//--Счетчик принятых успешных транзакций; 
uint16_t qntByte;
//-----------------------------------------
uint8_t  dp=0;					//--Номер текущей страницы;
uint8_t  buff[RT_HW_NEXT_BUFF_REG_SIZE];	//--Буфер для приема данных      из принятой транзакции;
//-----------------------------------------
uint8_t  freshHead;             //--Флаг принятого байта до кода заголовка (0x3A);
uint8_t  codeGetHead;			//--Код  принятого байта до кода заголовка (0x3A);
//--------------------------------------------
//uint8_t  maxPack=3;				//--Максимальное кол-во одновременного отправляемых транзакций;
//uint8_t  cntWatchDogBusy;
//uint32_t timePack;				//--Переменная для работы со временем тайм-аута контроля посылок;
//uint16_t periodPack;			//--Расчетное время на отправку одной транзакции; 
//uint8_t  cntReboot;				//-- 
}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	СТРУКТУРА RT_HW_STRUCT_NEXT_WRITE_VAL- параметры устройства ;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_STRUCT_NEXT_WRITE_VAL{		//==Структура для записи в Nextion целого значения с именем inx;
//-----------------Параметр используемый для отладки-----------------------------------------------
//uint8_t  debug=0;					//--Код состояния отладки. Изменяется внешними функциями;
//-----------------Параметры корректируемые до вызова функции setID()------------------------------
uint8_t  tick=2;				//--Период в тиках;
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  num=0; 				//--Номер устройства (от 1 до MAX_DEVICE).По этому номеру внешние блоки привязываются к устройству;
uint8_t  custom=0;				//--Код настройки; 
uint8_t  dir=0;					//--Этапы основной программы управления устройствами;
uint8_t  adr=0;					//--Адрес записываемого регистра;
uint8_t  run=1;					//--Флаг готовности данных к отправке в Nextion;
uint8_t  agoBlink;				//--Предыдущее состояние 
uint8_t  cnt=0;					//--Счетчик для вычисления периода (в тикак); 
//uint8_t  stepDelay=0;
//uint32_t timeBegin;
int32_t  agoVal;				//--Предыдущее значение выводимой переменной;		   
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    Структура данных рабочих параметров
// set - флаги занятости устройства
// qnt - кол-во занятых устройств
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_NEXT_HIVE	{
uint16_t set;						//--Флаги занятости устройства 
uint8_t  qnt; 						//--Кол-во зарегистрированных устройств;
uint8_t  arrByte[2]={0,0};			//--1-байтовый массив для недопустимых устройств;
uint16_t arrWord[1]={0};			//--2-байтовый массив для недопустимых устройств;
uint32_t arrLong[1]={0};			//--4-байтовый массив для недопустимых устройств;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		КЛАСС: RT_HW_NEXT
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_NEXT{
public:
//------Данные-------------------------------------------------------------------------------------
RT_HW_NEXT_HIVE	hive;									//==Общие данные для работы библиотеки;
RT_HW_STRUCT_NEXT_DEVICE *ptr[RT_HW_NEXT_MAX_DEVICE+1];	//==Массив указателей на данные со структурой устройств;
RT_HW_STRUCT_NEXT_DEVICE zN;							//==Данные для копирования в не инициализированные указатели;
RT_HW_NEXT();											//==Конструктор;
//------Функции блока управления устройством-------------------------------------------------------	
void 	 setID 	   (RT_HW_STRUCT_NEXT_DEVICE &z,uint8_t *arrByte,uint16_t *arrWord,uint32_t *arrLong, uint8_t lenArrByte,uint8_t lenArrWord, uint8_t lenArrLong);									  
void 	 generator (RT_HW_STRUCT_NEXT_DEVICE &z);		//==generator tick; 
void 	 direct    (RT_HW_STRUCT_NEXT_DEVICE &z);		//==Управление общее;
//------Функции блока приема данных из Nextion------------------------------------------------------
void 	 getFromNxt(RT_HW_STRUCT_NEXT_DEVICE &z);		//==Получение пакета;
void 	 packFromRX(RT_HW_STRUCT_NEXT_DEVICE &z);		//==Вывод принятого пакета;
//void 	 getFromNxt_1(RT_HW_STRUCT_NEXT_DEVICE &z);
//------Функции блока отправки данных и инструкций в Nextion----------------------------------------
void 	 setValID   (RT_HW_STRUCT_NEXT_WRITE_VAL &id);								  
void 	 sendVal    (RT_HW_STRUCT_NEXT_WRITE_VAL &id, uint8_t adr, int32_t val);
void 	 sendNameVal(RT_HW_STRUCT_NEXT_WRITE_VAL &id, String name, int32_t val);
void 	 sendInst   (RT_HW_STRUCT_NEXT_WRITE_VAL &id, String inst);
//------Функции для доступа к данным устройства по его номеру (num)---------------------------------
bool 	 checkNum	 (uint8_t num);								//==check наличия устройства с номером num;
uint32_t getPtrArr   (uint8_t num, char mode);					//==getting адреса указателя на массив;
uint8_t  getLenArr   (uint8_t num, char mode);					//==getting длины массива;
uint32_t getValArr   (uint8_t num, char mode, uint8_t index);	//==getting значение из массива;
void  	 setValArr   (uint8_t num, char mode, uint8_t index, uint32_t val);	//==setting значение в массиве;
//------Проверка и управления флагом занятости устройством--------------------------------------------
bool    checkBusy   (uint8_t num);
//bool    checkSetBusy(uint8_t num);
void    setBusy     (uint8_t num);
void    setBusy     (uint8_t num, uint8_t len);
//void    clearBusy   (uint8_t num);
void  setLenSendPack(uint8_t num, uint8_t len);
//=================================================================================================
};  //---END 
//==============  Конец класса =====================================================================
extern RT_HW_NEXT RT_HW_Next; //--Создание объекта 
//===================================================================================
#endif //---RT_HW_NEXT_h



